% PT2PT_3D_COVARIANCE_CLOSEDFORM 
% Closed form solution for ICP covariance
%
%% Description:
%
% Author - 	
%           Sergi Foix Salmeron
%           PRE-JAE
%           Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 03/2011
%
% Usage:
%   [ CovX ] = pt2pt_3D_Covariance_closedForm( refPointCloud,
%   newPointCloud, xyzRxRyRz, CovZ )
%
% Variables:
%   Inputs:
%       refPointCloud   - a nx3 matrix, data set 1
%       newPointCloud   - a nx3 matrix, data set 2
%       xyzRxRyRz       -  a 1x6 vector, robot pose
%       CovZ            - a 3nx3n matrix, data set 1 points covariance
%
% 	Outputs:
%		CovX	- a 6x61 matrix, related pose covariance
%
%

%% Code:
function [ CovX ] = pt2pt_3D_Covariance_closedForm( refPointCloud, newPointCloud, xyzRxRyRz, CovZ )
% Estimacio de la transformacio
tx      = xyzRxRyRz(1);
ty      = xyzRxRyRz(2);
tz      = xyzRxRyRz(3);
psi     = xyzRxRyRz(4); % rx
theta   = xyzRxRyRz(5); % ry
phi     = xyzRxRyRz(6); % rz

EHess = zeros(6,6);

c.m = refPointCloud;
c.s = newPointCloud;
n = size(c.s,2);
d2J_dposedx_z = zeros(6,n);
d2J_dposedy_z = zeros(6,n);
d2J_dposedz_z = zeros(6,n);

c_phi = cos(phi);
c_theta = cos(theta);
c_psi = cos(psi);
s_phi = sin(phi);
s_theta = sin(theta);
s_psi = sin(psi);

s_psi_s_phi = s_psi*s_phi;
s_psi_s_theta= s_psi*s_theta;
sin_theta_s_phi =s_theta*s_phi;

s_theta_c_phi = s_theta*c_phi;
s_psi_c_phi =s_psi*c_phi;
s_psi_c_theta = s_psi*c_theta;


c_theta_c_phi = c_theta*c_phi;
c_psi_c_phi =c_psi*c_phi;

c_psi_s_phi =c_psi*s_phi;
c_psi_c_theta = c_psi*c_theta;
c_psi_s_theta =c_psi*s_theta;

d2J_dtxdx_z = 2*c_psi*c_theta;
d2J_dtydx_z = 2*s_psi*c_theta;
d2J_dtzdx_z = -2*s_theta;


d2J_dtxdy_z = -2*(s_psi*c_phi+c_psi*s_theta*s_phi);
d2J_dtydy_z = 2*(c_psi*c_phi+s_psi*s_theta*s_phi);
d2J_dtzdy_z = 2*c_theta*s_phi;

d2J_dtxdz_z = 2*(s_psi*s_phi+c_psi*s_theta*c_phi);
d2J_dtydz_z = -2*(c_psi*s_phi+s_psi*s_theta*c_phi);
d2J_dtzdz_z = 2*c_theta*c_phi;

for ii=1:n
%     fprintf('%d\n',ii);

    x_z = c.s(1,ii);
    y_z = c.s(2,ii);
    z_z = c.s(3,ii);
    x_m = c.m(1,ii);
    y_m = c.m(2,ii);
    z_m = c.m(3,ii);
    
    %% Calcul del Jacobia de la funcio de minimitzacio ICP
    
%     dJ_dtx = 2*c_psi_c_theta*x_z+2*(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+2*(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+2*tx-2*x_m;
%     
%     dJ_dty = 2*s_psi_c_theta*x_z+2*(c_psi*c_phi+s_psi_s_theta*s_phi)*y_z+2*(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+2*ty-2*y_m;
%         
%     dJ_dtz = -2*s_theta*x_z+2*c_theta*s_phi*y_z+2*c_theta_c_phi*z_z+2*tz-2*z_m;
%     
%     dJ_dphi = 2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*((s_psi_s_phi+c_psi*s_theta_c_phi)*y_z+(s_psi_c_phi-c_psi_s_theta*s_phi)*z_z)+2*(s_psi_c_theta*x_z+(c_psi*c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*((-c_psi_s_phi+s_psi*s_theta_c_phi)*y_z+(-c_psi*c_phi-s_psi_s_theta*s_phi)*z_z)+2*(-s_theta*x_z+c_theta*s_phi*y_z+c_theta_c_phi*z_z+tz-z_m)*(c_theta_c_phi*y_z-c_theta*s_phi*z_z);
%         
%     dJ_dtheta = 2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*(-c_psi_s_theta*x_z+c_psi_c_theta*s_phi*y_z+c_psi_c_theta*c_phi*z_z)+2*(s_psi_c_theta*x_z+(c_psi*c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*(-s_psi_s_theta*x_z+s_psi_c_theta*s_phi*y_z+s_psi_c_theta*c_phi*z_z)+2*(-s_theta*x_z+c_theta*s_phi*y_z+c_theta_c_phi*z_z+tz-z_m)*(-c_theta*x_z-sin_theta_s_phi*y_z-s_theta_c_phi*z_z);
%        
%     dJ_dpsi = 2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*(-s_psi_c_theta*x_z+(-c_psi*c_phi-s_psi_s_theta*s_phi)*y_z+(c_psi_s_phi-s_psi*s_theta_c_phi)*z_z)+2*(s_psi_c_theta*x_z+(c_psi*c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z);
    
    
    Hess = [ 2, 0, 0,2*((s_psi_s_phi+c_psi*s_theta_c_phi)*y_z+(s_psi_c_phi-c_psi_s_theta*s_phi)*z_z),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                -2*c_psi_s_theta*x_z+2*c_psi_c_theta*s_phi*y_z+2*c_psi_c_theta*c_phi*z_z,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       -2*s_psi_c_theta*x_z+2*(-c_psi*c_phi-s_psi_s_theta*s_phi)*y_z+2*(c_psi_s_phi-s_psi*s_theta_c_phi)*z_z;
             0, 2, 0,2*((-c_psi_s_phi+s_psi*s_theta_c_phi)*y_z+(-c_psi*c_phi-s_psi_s_theta*s_phi)*z_z),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                -2*s_psi_s_theta*x_z+2*s_psi_c_theta*s_phi*y_z+2*s_psi_c_theta*c_phi*z_z,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        2*c_psi_c_theta*x_z+2*(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+2*(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z;
             0, 0, 2,2*(c_theta_c_phi*y_z-c_theta*s_phi*z_z),-2*c_theta*x_z-2*sin_theta_s_phi*y_z-2*s_theta_c_phi*z_z,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               0;
             2*(s_psi_s_phi+c_psi*s_theta_c_phi)*y_z+2*(s_psi_c_phi-c_psi_s_theta*s_phi)*z_z,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 2*(-c_psi_s_phi+s_psi*s_theta_c_phi)*y_z+2*(-c_psi*c_phi-s_psi_s_theta*s_phi)*z_z,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             2*c_theta_c_phi*y_z-2*c_theta*s_phi*z_z,                                                                                                                                                                     2*((s_psi_s_phi+c_psi*s_theta_c_phi)*y_z+(s_psi_c_phi-c_psi_s_theta*s_phi)*z_z)^2+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*((s_psi_c_phi-c_psi_s_theta*s_phi)*y_z+(-s_psi_s_phi-c_psi*s_theta_c_phi)*z_z)+2*((-c_psi_s_phi+s_psi*s_theta_c_phi)*y_z+(-c_psi*c_phi-s_psi_s_theta*s_phi)*z_z)^2+2*(s_psi_c_theta*x_z+(c_psi*c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*((-c_psi*c_phi-s_psi_s_theta*s_phi)*y_z+(c_psi_s_phi-s_psi*s_theta_c_phi)*z_z)+2*(c_theta_c_phi*y_z-c_theta*s_phi*z_z)^2+2*(-s_theta*x_z+c_theta*s_phi*y_z+c_theta_c_phi*z_z+tz-z_m)*(-c_theta*s_phi*y_z-c_theta_c_phi*z_z), 2*(-c_psi_s_theta*x_z+c_psi_c_theta*s_phi*y_z+c_psi_c_theta*c_phi*z_z)*((s_psi_s_phi+c_psi*s_theta_c_phi)*y_z+(s_psi_c_phi-c_psi_s_theta*s_phi)*z_z)+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*(c_psi_c_theta*c_phi*y_z-c_psi_c_theta*s_phi*z_z)+2*(-s_psi_s_theta*x_z+s_psi_c_theta*s_phi*y_z+s_psi_c_theta*c_phi*z_z)*((-c_psi_s_phi+s_psi*s_theta_c_phi)*y_z+(-c_psi*c_phi-s_psi_s_theta*s_phi)*z_z)+2*(s_psi_c_theta*x_z+(c_psi*c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*(s_psi_c_theta*c_phi*y_z-s_psi_c_theta*s_phi*z_z)+2*(-c_theta*x_z-sin_theta_s_phi*y_z-s_theta_c_phi*z_z)*(c_theta_c_phi*y_z-c_theta*s_phi*z_z)+2*(-s_theta*x_z+c_theta*s_phi*y_z+c_theta_c_phi*z_z+tz-z_m)*(-s_theta_c_phi*y_z+sin_theta_s_phi*z_z),                                                                                    2*(-s_psi_c_theta*x_z+(-c_psi*c_phi-s_psi_s_theta*s_phi)*y_z+(c_psi_s_phi-s_psi*s_theta_c_phi)*z_z)*((s_psi_s_phi+c_psi*s_theta_c_phi)*y_z+(s_psi_c_phi-c_psi_s_theta*s_phi)*z_z)+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*((c_psi_s_phi-s_psi*s_theta_c_phi)*y_z+(c_psi*c_phi+s_psi_s_theta*s_phi)*z_z)+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z)*((-c_psi_s_phi+s_psi*s_theta_c_phi)*y_z+(-c_psi*c_phi-s_psi_s_theta*s_phi)*z_z)+2*(s_psi_c_theta*x_z+(c_psi*c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*((s_psi_s_phi+c_psi*s_theta_c_phi)*y_z+(s_psi_c_phi-c_psi_s_theta*s_phi)*z_z);
             -2*(c_psi_s_theta*x_z+c_psi_c_theta*s_phi*y_z+c_psi_c_theta*c_phi*z_z),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                -2*s_psi_s_theta*x_z+2*s_psi_c_theta*s_phi*y_z+2*s_psi_c_theta*c_phi*z_z,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           -2*c_theta*x_z-2*sin_theta_s_phi*y_z-2*s_theta_c_phi*z_z, 2*(-c_psi_s_theta*x_z+c_psi_c_theta*s_phi*y_z+c_psi_c_theta*c_phi*z_z)*((s_psi_s_phi+c_psi*s_theta_c_phi)*y_z+(s_psi_c_phi-c_psi_s_theta*s_phi)*z_z)+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*(c_psi_c_theta*c_phi*y_z-c_psi_c_theta*s_phi*z_z)+2*(-s_psi_s_theta*x_z+s_psi_c_theta*s_phi*y_z+s_psi_c_theta*c_phi*z_z)*((-c_psi_s_phi+s_psi*s_theta_c_phi)*y_z+(-c_psi*c_phi-s_psi_s_theta*s_phi)*z_z)+2*(s_psi_c_theta*x_z+(c_psi*c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*(s_psi_c_theta*c_phi*y_z-s_psi_c_theta*s_phi*z_z)+2*(-c_theta*x_z-sin_theta_s_phi*y_z-s_theta_c_phi*z_z)*(c_theta_c_phi*y_z-c_theta*s_phi*z_z)+2*(-s_theta*x_z+c_theta*s_phi*y_z+c_theta_c_phi*z_z+tz-z_m)*(-s_theta_c_phi*y_z+sin_theta_s_phi*z_z),                                                                                                                                                                                                       2*(-c_psi_s_theta*x_z+c_psi_c_theta*s_phi*y_z+c_psi_c_theta*c_phi*z_z)^2+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*(-c_psi_c_theta*x_z-c_psi_s_theta*s_phi*y_z-c_psi*s_theta_c_phi*z_z)+2*(-s_psi_s_theta*x_z+s_psi_c_theta*s_phi*y_z+s_psi_c_theta*c_phi*z_z)^2+2*(s_psi_c_theta*x_z+(c_psi*c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*(-s_psi_c_theta*x_z-s_psi_s_theta*s_phi*y_z-s_psi*s_theta_c_phi*z_z)+2*(-c_theta*x_z-sin_theta_s_phi*y_z-s_theta_c_phi*z_z)^2+2*(-s_theta*x_z+c_theta*s_phi*y_z+c_theta_c_phi*z_z+tz-z_m)*(s_theta*x_z-c_theta*s_phi*y_z-c_theta_c_phi*z_z),                                                                                                                                                   2*(-s_psi_c_theta*x_z+(-c_psi*c_phi-s_psi_s_theta*s_phi)*y_z+(c_psi_s_phi-s_psi*s_theta_c_phi)*z_z)*(-c_psi_s_theta*x_z+c_psi_c_theta*s_phi*y_z+c_psi_c_theta*c_phi*z_z)+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*(s_psi_s_theta*x_z-s_psi_c_theta*s_phi*y_z-s_psi_c_theta*c_phi*z_z)+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z)*(-s_psi_s_theta*x_z+s_psi_c_theta*s_phi*y_z+s_psi_c_theta*c_phi*z_z)+2*(s_psi_c_theta*x_z+(c_psi*c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*(-c_psi_s_theta*x_z+c_psi_c_theta*s_phi*y_z+c_psi_c_theta*c_phi*z_z);
             -2*(s_psi_c_theta*x_z+(-c_psi*c_phi-s_psi_s_theta*s_phi)*y_z+(c_psi_s_phi-s_psi*s_theta_c_phi)*z_z),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        2*c_psi_c_theta*x_z+2*(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+2*(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               0,                                                                                    2*(-s_psi_c_theta*x_z+(-c_psi*c_phi-s_psi_s_theta*s_phi)*y_z+(c_psi_s_phi-s_psi*s_theta_c_phi)*z_z)*((s_psi_s_phi+c_psi*s_theta_c_phi)*y_z+(s_psi_c_phi-c_psi_s_theta*s_phi)*z_z)+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*((c_psi_s_phi-s_psi*s_theta_c_phi)*y_z+(c_psi*c_phi+s_psi_s_theta*s_phi)*z_z)+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z)*((-c_psi_s_phi+s_psi*s_theta_c_phi)*y_z+(-c_psi*c_phi-s_psi_s_theta*s_phi)*z_z)+2*(s_psi_c_theta*x_z+(c_psi*c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*((s_psi_s_phi+c_psi*s_theta_c_phi)*y_z+(s_psi_c_phi-c_psi_s_theta*s_phi)*z_z),                                                                                                                                                   2*(-s_psi_c_theta*x_z+(-c_psi*c_phi-s_psi_s_theta*s_phi)*y_z+(c_psi_s_phi-s_psi*s_theta_c_phi)*z_z)*(-c_psi_s_theta*x_z+c_psi_c_theta*s_phi*y_z+c_psi_c_theta*c_phi*z_z)+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*(s_psi_s_theta*x_z-s_psi_c_theta*s_phi*y_z-s_psi_c_theta*c_phi*z_z)+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z)*(-s_psi_s_theta*x_z+s_psi_c_theta*s_phi*y_z+s_psi_c_theta*c_phi*z_z)+2*(s_psi_c_theta*x_z+(c_psi*c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*(-c_psi_s_theta*x_z+c_psi_c_theta*s_phi*y_z+c_psi_c_theta*c_phi*z_z),                                                                                                                                                                                                                                                      2*(-s_psi_c_theta*x_z+(-c_psi*c_phi-s_psi_s_theta*s_phi)*y_z+(c_psi_s_phi-s_psi*s_theta_c_phi)*z_z)^2+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*(-c_psi_c_theta*x_z+(s_psi_c_phi-c_psi_s_theta*s_phi)*y_z+(-s_psi_s_phi-c_psi*s_theta_c_phi)*z_z)+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z)^2+2*(s_psi_c_theta*x_z+(c_psi*c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*(-s_psi_c_theta*x_z+(-c_psi*c_phi-s_psi_s_theta*s_phi)*y_z+(c_psi_s_phi-s_psi*s_theta_c_phi)*z_z)];

    % Sumatori per a tots els punts
    EHess = EHess + Hess;
    
    % Calcul de les derivades parcials respecte les observacions sobre el Jacobia de la funcio de min.
    
    d2J_dphidx_z = 2*c_psi_c_theta*((s_psi_s_phi+c_psi*s_theta_c_phi)*y_z+(s_psi_c_phi-c_psi_s_theta*s_phi)*z_z)+2*s_psi_c_theta*((-c_psi_s_phi+s_psi*s_theta_c_phi)*y_z+(-c_psi*c_phi-s_psi_s_theta*s_phi)*z_z)-2*s_theta*(c_theta_c_phi*y_z-c_theta*s_phi*z_z);
    d2J_dthetadx_z = 2*c_psi_c_theta*(-c_psi_s_theta*x_z+c_psi_c_theta*s_phi*y_z+c_psi_c_theta*c_phi*z_z)-2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*c_psi_s_theta+2*s_psi_c_theta*(-s_psi_s_theta*x_z+s_psi_c_theta*s_phi*y_z+s_psi_c_theta*c_phi*z_z)-2*(s_psi_c_theta*x_z+(c_psi*c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*s_psi_s_theta-2*s_theta*(-c_theta*x_z-sin_theta_s_phi*y_z-s_theta_c_phi*z_z)-2*(-s_theta*x_z+c_theta*s_phi*y_z+c_theta_c_phi*z_z+tz-z_m)*c_theta;
    d2J_dpsidx_z = 2*c_psi_c_theta*(-s_psi_c_theta*x_z+(-c_psi_c_phi-s_psi_s_theta*s_phi)*y_z+(c_psi_s_phi-s_psi*s_theta_c_phi)*z_z)-2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*s_psi_c_theta+2*s_psi_c_theta*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z)+2*(s_psi_c_theta*x_z+(c_psi_c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*c_psi_c_theta;

    d2J_dphidy_z = 2*(-s_psi_c_phi+c_psi_s_theta*s_phi)*((s_psi_s_phi+c_psi*s_theta_c_phi)*y_z+(s_psi_c_phi-c_psi_s_theta*s_phi)*z_z)+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*(s_psi_s_phi+c_psi*s_theta_c_phi)+2*(c_psi_c_phi+s_psi_s_theta*s_phi)*((-c_psi_s_phi+s_psi*s_theta_c_phi)*y_z+(-c_psi_c_phi-s_psi_s_theta*s_phi)*z_z)+2*(s_psi_c_theta*x_z+(c_psi_c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*(-c_psi_s_phi+s_psi*s_theta_c_phi)+2*c_theta*s_phi*(c_theta_c_phi*y_z-c_theta*s_phi*z_z)+2*(-s_theta*x_z+c_theta*s_phi*y_z+c_theta_c_phi*z_z+tz-z_m)*c_theta_c_phi;
    d2J_dthetady_z = 2*(-s_psi_c_phi+c_psi_s_theta*s_phi)*(-c_psi_s_theta*x_z+c_psi_c_theta*s_phi*y_z+c_psi_c_theta*c_phi*z_z)+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*c_psi_c_theta*s_phi+2*(c_psi_c_phi+s_psi_s_theta*s_phi)*(-s_psi_s_theta*x_z+s_psi_c_theta*s_phi*y_z+s_psi_c_theta*c_phi*z_z)+2*(s_psi_c_theta*x_z+(c_psi_c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*s_psi_c_theta*s_phi+2*c_theta*s_phi*(-c_theta*x_z-sin_theta_s_phi*y_z-s_theta_c_phi*z_z)-2*(-s_theta*x_z+c_theta*s_phi*y_z+c_theta_c_phi*z_z+tz-z_m)*sin_theta_s_phi;
    d2J_dpsidy_z = 2*(-s_psi_c_phi+c_psi_s_theta*s_phi)*(-s_psi_c_theta*x_z+(-c_psi_c_phi-s_psi_s_theta*s_phi)*y_z+(c_psi_s_phi-s_psi*s_theta_c_phi)*z_z)+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*(-c_psi_c_phi-s_psi_s_theta*s_phi)+2*(c_psi_c_phi+s_psi_s_theta*s_phi)*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z)+2*(s_psi_c_theta*x_z+(c_psi_c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*(-s_psi_c_phi+c_psi_s_theta*s_phi);
    
    d2J_dphidz_z = 2*(s_psi_s_phi+c_psi*s_theta_c_phi)*((s_psi_s_phi+c_psi*s_theta_c_phi)*y_z+(s_psi_c_phi-c_psi_s_theta*s_phi)*z_z)+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*(s_psi_c_phi-c_psi_s_theta*s_phi)+2*(-c_psi_s_phi+s_psi*s_theta_c_phi)*((-c_psi_s_phi+s_psi*s_theta_c_phi)*y_z+(-c_psi_c_phi-s_psi_s_theta*s_phi)*z_z)+2*(s_psi_c_theta*x_z+(c_psi_c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*(-c_psi_c_phi-s_psi_s_theta*s_phi)+2*c_theta_c_phi*(c_theta_c_phi*y_z-c_theta*s_phi*z_z)-2*(-s_theta*x_z+c_theta*s_phi*y_z+c_theta_c_phi*z_z+tz-z_m)*c_theta*s_phi;
    d2J_dthetadz_z = 2*(s_psi_s_phi+c_psi*s_theta_c_phi)*(-c_psi_s_theta*x_z+c_psi_c_theta*s_phi*y_z+c_psi_c_theta*c_phi*z_z)+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*c_psi_c_theta*c_phi+2*(-c_psi_s_phi+s_psi*s_theta_c_phi)*(-s_psi_s_theta*x_z+s_psi_c_theta*s_phi*y_z+s_psi_c_theta*c_phi*z_z)+2*(s_psi_c_theta*x_z+(c_psi_c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*s_psi_c_theta*c_phi+2*c_theta_c_phi*(-c_theta*x_z-sin_theta_s_phi*y_z-s_theta_c_phi*z_z)-2*(-s_theta*x_z+c_theta*s_phi*y_z+c_theta_c_phi*z_z+tz-z_m)*s_theta_c_phi;
    d2J_dpsidz_z = 2*(s_psi_s_phi+c_psi*s_theta_c_phi)*(-s_psi_c_theta*x_z+(-c_psi_c_phi-s_psi_s_theta*s_phi)*y_z+(c_psi_s_phi-s_psi*s_theta_c_phi)*z_z)+2*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z+tx-x_m)*(c_psi_s_phi-s_psi*s_theta_c_phi)+2*(-c_psi_s_phi+s_psi*s_theta_c_phi)*(c_psi_c_theta*x_z+(-s_psi_c_phi+c_psi_s_theta*s_phi)*y_z+(s_psi_s_phi+c_psi*s_theta_c_phi)*z_z)+2*(s_psi_c_theta*x_z+(c_psi_c_phi+s_psi_s_theta*s_phi)*y_z+(-c_psi_s_phi+s_psi*s_theta_c_phi)*z_z+ty-y_m)*(s_psi_s_phi+c_psi*s_theta_c_phi);
    
    d2J_dposedx_z(:,ii) = [ d2J_dtxdx_z ; d2J_dtydx_z ; d2J_dtzdx_z ; d2J_dphidx_z ; d2J_dthetadx_z ; d2J_dpsidx_z ];
    d2J_dposedy_z(:,ii) = [ d2J_dtxdy_z ; d2J_dtydy_z ; d2J_dtzdy_z ; d2J_dphidy_z ; d2J_dthetady_z ; d2J_dpsidy_z ];
    d2J_dposedz_z(:,ii) =  [ d2J_dtxdz_z ; d2J_dtydz_z ; d2J_dtzdz_z ; d2J_dphidz_z ; d2J_dthetadz_z ; d2J_dpsidz_z ];
    

end

E2J_dposedz = [ d2J_dposedx_z  d2J_dposedy_z d2J_dposedz_z ];

Jacobian = EHess\E2J_dposedz;

% CovX = inv(EHess)*E2J_dposedz * CovZ * E2J_dposedz'*inv(EHess);
CovX = Jacobian * CovZ * Jacobian';
