%% pt2pt_3D_Covariance_closedForm (Close Form Derived)
%  Funcio que retorna la covariance de la estimacio de la pose d'un ICP
% 
% CovX = pt2pt_3D_Covariance_closedForm( refPointCloud, newPointCloud, xyzRxRyRz, CovZ )
% 
% Inputs:
%   - refPointCloud   -> Reference Point Cloud Correspondences
%   - newPointCloud   -> New Point Cloud Correspondences
%   - xyzRxRyRz       -> [ tx ty tz Rx Ry Rz ]', estimated pose
%   - Covz  -> Sensor readings covariance
% 
% Output:
%   - CovX   -> ICP3D Covariance Matrix [6x6]
% 
% Modified:
%   - Sergi Foix Salmerón
%   - PhD Student at IRI

%% code

function [ CovX ] = pt2pt_3D_Covariance_closedForm_o( refPointCloud, newPointCloud, xyzRxRyRz, CovZ )
% print info
% fprintf(' -> pt2pt Cov Closed Form Derived \n');

% Estimacio de la transformacio
tx      = xyzRxRyRz(1);
ty      = xyzRxRyRz(2);
tz      = xyzRxRyRz(3);
phi     = xyzRxRyRz(4);
theta   = xyzRxRyRz(5);
psi     = xyzRxRyRz(6);

EHess = zeros(6,6);

c.m = refPointCloud;
c.s = newPointCloud;

d2J_dposedx_z = zeros(6,size(c.s,2));
d2J_dposedy_z = zeros(6,size(c.s,2));
d2J_dposedz_z = zeros(6,size(c.s,2));
a=1;
for ii=1:size(c.s,2)
%     if rem(a,10)
%         fprintf('* ');
%     else
%         fprintf('*\n');
%     end
    a=a+1;

    x_z = c.s(1,ii);
    y_z = c.s(2,ii);
    z_z = c.s(3,ii);
    x_m = c.m(1,ii);
    y_m = c.m(2,ii);
    z_m = c.m(3,ii);
    
    %% Calcul del Jacobia de la funcio de minimitzacio ICP
%     First derivatives
%     dJ_dtx = 2*cos(psi)*cos(theta)*x_z+2*(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+2*(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+2*tx-2*x_m;
%     
%     dJ_dty = 2*sin(psi)*cos(theta)*x_z+2*(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+2*(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+2*ty-2*y_m;
%         
%     dJ_dtz = -2*sin(theta)*x_z+2*cos(theta)*sin(phi)*y_z+2*cos(theta)*cos(phi)*z_z+2*tz-2*z_m;
%     
%     dJ_dphi = 2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*((sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*y_z+(sin(psi)*cos(phi)-cos(psi)*sin(theta)*sin(phi))*z_z)+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*((-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*y_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*z_z)+2*(-sin(theta)*x_z+cos(theta)*sin(phi)*y_z+cos(theta)*cos(phi)*z_z+tz-z_m)*(cos(theta)*cos(phi)*y_z-cos(theta)*sin(phi)*z_z);
%         
%     dJ_dtheta = 2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*(-cos(psi)*sin(theta)*x_z+cos(psi)*cos(theta)*sin(phi)*y_z+cos(psi)*cos(theta)*cos(phi)*z_z)+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*(-sin(psi)*sin(theta)*x_z+sin(psi)*cos(theta)*sin(phi)*y_z+sin(psi)*cos(theta)*cos(phi)*z_z)+2*(-sin(theta)*x_z+cos(theta)*sin(phi)*y_z+cos(theta)*cos(phi)*z_z+tz-z_m)*(-cos(theta)*x_z-sin(theta)*sin(phi)*y_z-sin(theta)*cos(phi)*z_z);
%        
%     dJ_dpsi = 2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*(-sin(psi)*cos(theta)*x_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*y_z+(cos(psi)*sin(phi)-sin(psi)*sin(theta)*cos(phi))*z_z)+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z);
%     
    
    Hess = [ 2, 0, 0, 2*(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*y_z+2*(sin(psi)*cos(phi)-cos(psi)*sin(theta)*sin(phi))*z_z,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                -2*cos(psi)*sin(theta)*x_z+2*cos(psi)*cos(theta)*sin(phi)*y_z+2*cos(psi)*cos(theta)*cos(phi)*z_z,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       -2*sin(psi)*cos(theta)*x_z+2*(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*y_z+2*(cos(psi)*sin(phi)-sin(psi)*sin(theta)*cos(phi))*z_z;
             0, 2, 0, 2*(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*y_z+2*(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*z_z,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                -2*sin(psi)*sin(theta)*x_z+2*sin(psi)*cos(theta)*sin(phi)*y_z+2*sin(psi)*cos(theta)*cos(phi)*z_z,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        2*cos(psi)*cos(theta)*x_z+2*(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+2*(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z;
             0, 0, 2, 2*cos(theta)*cos(phi)*y_z-2*cos(theta)*sin(phi)*z_z,-2*cos(theta)*x_z-2*sin(theta)*sin(phi)*y_z-2*sin(theta)*cos(phi)*z_z,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               0;
             2*(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*y_z+2*(sin(psi)*cos(phi)-cos(psi)*sin(theta)*sin(phi))*z_z,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 2*(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*y_z+2*(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*z_z,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             2*cos(theta)*cos(phi)*y_z-2*cos(theta)*sin(phi)*z_z,                                                                                                                                                                     2*((sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*y_z+(sin(psi)*cos(phi)-cos(psi)*sin(theta)*sin(phi))*z_z)^2+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*((sin(psi)*cos(phi)-cos(psi)*sin(theta)*sin(phi))*y_z+(-sin(psi)*sin(phi)-cos(psi)*sin(theta)*cos(phi))*z_z)+2*((-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*y_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*z_z)^2+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*((-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*y_z+(cos(psi)*sin(phi)-sin(psi)*sin(theta)*cos(phi))*z_z)+2*(cos(theta)*cos(phi)*y_z-cos(theta)*sin(phi)*z_z)^2+2*(-sin(theta)*x_z+cos(theta)*sin(phi)*y_z+cos(theta)*cos(phi)*z_z+tz-z_m)*(-cos(theta)*sin(phi)*y_z-cos(theta)*cos(phi)*z_z), 2*(-cos(psi)*sin(theta)*x_z+cos(psi)*cos(theta)*sin(phi)*y_z+cos(psi)*cos(theta)*cos(phi)*z_z)*((sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*y_z+(sin(psi)*cos(phi)-cos(psi)*sin(theta)*sin(phi))*z_z)+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*(cos(psi)*cos(theta)*cos(phi)*y_z-cos(psi)*cos(theta)*sin(phi)*z_z)+2*(-sin(psi)*sin(theta)*x_z+sin(psi)*cos(theta)*sin(phi)*y_z+sin(psi)*cos(theta)*cos(phi)*z_z)*((-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*y_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*z_z)+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*(sin(psi)*cos(theta)*cos(phi)*y_z-sin(psi)*cos(theta)*sin(phi)*z_z)+2*(-cos(theta)*x_z-sin(theta)*sin(phi)*y_z-sin(theta)*cos(phi)*z_z)*(cos(theta)*cos(phi)*y_z-cos(theta)*sin(phi)*z_z)+2*(-sin(theta)*x_z+cos(theta)*sin(phi)*y_z+cos(theta)*cos(phi)*z_z+tz-z_m)*(-sin(theta)*cos(phi)*y_z+sin(theta)*sin(phi)*z_z),                                                                                    2*(-sin(psi)*cos(theta)*x_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*y_z+(cos(psi)*sin(phi)-sin(psi)*sin(theta)*cos(phi))*z_z)*((sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*y_z+(sin(psi)*cos(phi)-cos(psi)*sin(theta)*sin(phi))*z_z)+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*((cos(psi)*sin(phi)-sin(psi)*sin(theta)*cos(phi))*y_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*z_z)+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z)*((-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*y_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*z_z)+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*((sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*y_z+(sin(psi)*cos(phi)-cos(psi)*sin(theta)*sin(phi))*z_z);
            -2*cos(psi)*sin(theta)*x_z+2*cos(psi)*cos(theta)*sin(phi)*y_z+2*cos(psi)*cos(theta)*cos(phi)*z_z,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                -2*sin(psi)*sin(theta)*x_z+2*sin(psi)*cos(theta)*sin(phi)*y_z+2*sin(psi)*cos(theta)*cos(phi)*z_z,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           -2*cos(theta)*x_z-2*sin(theta)*sin(phi)*y_z-2*sin(theta)*cos(phi)*z_z, 2*(-cos(psi)*sin(theta)*x_z+cos(psi)*cos(theta)*sin(phi)*y_z+cos(psi)*cos(theta)*cos(phi)*z_z)*((sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*y_z+(sin(psi)*cos(phi)-cos(psi)*sin(theta)*sin(phi))*z_z)+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*(cos(psi)*cos(theta)*cos(phi)*y_z-cos(psi)*cos(theta)*sin(phi)*z_z)+2*(-sin(psi)*sin(theta)*x_z+sin(psi)*cos(theta)*sin(phi)*y_z+sin(psi)*cos(theta)*cos(phi)*z_z)*((-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*y_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*z_z)+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*(sin(psi)*cos(theta)*cos(phi)*y_z-sin(psi)*cos(theta)*sin(phi)*z_z)+2*(-cos(theta)*x_z-sin(theta)*sin(phi)*y_z-sin(theta)*cos(phi)*z_z)*(cos(theta)*cos(phi)*y_z-cos(theta)*sin(phi)*z_z)+2*(-sin(theta)*x_z+cos(theta)*sin(phi)*y_z+cos(theta)*cos(phi)*z_z+tz-z_m)*(-sin(theta)*cos(phi)*y_z+sin(theta)*sin(phi)*z_z),                                                                                                                                                                                                       2*(-cos(psi)*sin(theta)*x_z+cos(psi)*cos(theta)*sin(phi)*y_z+cos(psi)*cos(theta)*cos(phi)*z_z)^2+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*(-cos(psi)*cos(theta)*x_z-cos(psi)*sin(theta)*sin(phi)*y_z-cos(psi)*sin(theta)*cos(phi)*z_z)+2*(-sin(psi)*sin(theta)*x_z+sin(psi)*cos(theta)*sin(phi)*y_z+sin(psi)*cos(theta)*cos(phi)*z_z)^2+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*(-sin(psi)*cos(theta)*x_z-sin(psi)*sin(theta)*sin(phi)*y_z-sin(psi)*sin(theta)*cos(phi)*z_z)+2*(-cos(theta)*x_z-sin(theta)*sin(phi)*y_z-sin(theta)*cos(phi)*z_z)^2+2*(-sin(theta)*x_z+cos(theta)*sin(phi)*y_z+cos(theta)*cos(phi)*z_z+tz-z_m)*(sin(theta)*x_z-cos(theta)*sin(phi)*y_z-cos(theta)*cos(phi)*z_z),                                                                                                                                                   2*(-sin(psi)*cos(theta)*x_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*y_z+(cos(psi)*sin(phi)-sin(psi)*sin(theta)*cos(phi))*z_z)*(-cos(psi)*sin(theta)*x_z+cos(psi)*cos(theta)*sin(phi)*y_z+cos(psi)*cos(theta)*cos(phi)*z_z)+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*(sin(psi)*sin(theta)*x_z-sin(psi)*cos(theta)*sin(phi)*y_z-sin(psi)*cos(theta)*cos(phi)*z_z)+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z)*(-sin(psi)*sin(theta)*x_z+sin(psi)*cos(theta)*sin(phi)*y_z+sin(psi)*cos(theta)*cos(phi)*z_z)+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*(-cos(psi)*sin(theta)*x_z+cos(psi)*cos(theta)*sin(phi)*y_z+cos(psi)*cos(theta)*cos(phi)*z_z);
            -2*sin(psi)*cos(theta)*x_z+2*(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*y_z+2*(cos(psi)*sin(phi)-sin(psi)*sin(theta)*cos(phi))*z_z,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        2*cos(psi)*cos(theta)*x_z+2*(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+2*(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               0,                                                                                    2*(-sin(psi)*cos(theta)*x_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*y_z+(cos(psi)*sin(phi)-sin(psi)*sin(theta)*cos(phi))*z_z)*((sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*y_z+(sin(psi)*cos(phi)-cos(psi)*sin(theta)*sin(phi))*z_z)+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*((cos(psi)*sin(phi)-sin(psi)*sin(theta)*cos(phi))*y_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*z_z)+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z)*((-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*y_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*z_z)+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*((sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*y_z+(sin(psi)*cos(phi)-cos(psi)*sin(theta)*sin(phi))*z_z),                                                                                                                                                   2*(-sin(psi)*cos(theta)*x_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*y_z+(cos(psi)*sin(phi)-sin(psi)*sin(theta)*cos(phi))*z_z)*(-cos(psi)*sin(theta)*x_z+cos(psi)*cos(theta)*sin(phi)*y_z+cos(psi)*cos(theta)*cos(phi)*z_z)+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*(sin(psi)*sin(theta)*x_z-sin(psi)*cos(theta)*sin(phi)*y_z-sin(psi)*cos(theta)*cos(phi)*z_z)+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z)*(-sin(psi)*sin(theta)*x_z+sin(psi)*cos(theta)*sin(phi)*y_z+sin(psi)*cos(theta)*cos(phi)*z_z)+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*(-cos(psi)*sin(theta)*x_z+cos(psi)*cos(theta)*sin(phi)*y_z+cos(psi)*cos(theta)*cos(phi)*z_z),                                                                                                                                                                                                                                                      2*(-sin(psi)*cos(theta)*x_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*y_z+(cos(psi)*sin(phi)-sin(psi)*sin(theta)*cos(phi))*z_z)^2+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*(-cos(psi)*cos(theta)*x_z+(sin(psi)*cos(phi)-cos(psi)*sin(theta)*sin(phi))*y_z+(-sin(psi)*sin(phi)-cos(psi)*sin(theta)*cos(phi))*z_z)+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z)^2+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*(-sin(psi)*cos(theta)*x_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*y_z+(cos(psi)*sin(phi)-sin(psi)*sin(theta)*cos(phi))*z_z)];

        
    % Sumatori per a tots els punts
    EHess = EHess + Hess;
    
    % Calcul de les derivades parcials respecte les observacions sobre el Jacobia de la funcio de min.
    d2J_dtxdx_z = 2*cos(psi)*cos(theta);
    d2J_dtydx_z = 2*sin(psi)*cos(theta);
    d2J_dtzdx_z = -2*sin(theta);
    d2J_dphidx_z = 2*cos(psi)*cos(theta)*((sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*y_z+(sin(psi)*cos(phi)-cos(psi)*sin(theta)*sin(phi))*z_z)+2*sin(psi)*cos(theta)*((-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*y_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*z_z)-2*sin(theta)*(cos(theta)*cos(phi)*y_z-cos(theta)*sin(phi)*z_z);
    d2J_dthetadx_z = 2*cos(psi)*cos(theta)*(-cos(psi)*sin(theta)*x_z+cos(psi)*cos(theta)*sin(phi)*y_z+cos(psi)*cos(theta)*cos(phi)*z_z)-2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*cos(psi)*sin(theta)+2*sin(psi)*cos(theta)*(-sin(psi)*sin(theta)*x_z+sin(psi)*cos(theta)*sin(phi)*y_z+sin(psi)*cos(theta)*cos(phi)*z_z)-2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*sin(psi)*sin(theta)-2*sin(theta)*(-cos(theta)*x_z-sin(theta)*sin(phi)*y_z-sin(theta)*cos(phi)*z_z)-2*(-sin(theta)*x_z+cos(theta)*sin(phi)*y_z+cos(theta)*cos(phi)*z_z+tz-z_m)*cos(theta);
    d2J_dpsidx_z = 2*cos(psi)*cos(theta)*(-sin(psi)*cos(theta)*x_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*y_z+(cos(psi)*sin(phi)-sin(psi)*sin(theta)*cos(phi))*z_z)-2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*sin(psi)*cos(theta)+2*sin(psi)*cos(theta)*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z)+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*cos(psi)*cos(theta);
     
    d2J_dposedx_z_ii = [ d2J_dtxdx_z ; d2J_dtydx_z ; d2J_dtzdx_z ; d2J_dphidx_z ; d2J_dthetadx_z ; d2J_dpsidx_z ];

    d2J_dtxdy_z = -2*sin(psi)*cos(phi)+2*cos(psi)*sin(theta)*sin(phi);
    d2J_dtydy_z = 2*cos(psi)*cos(phi)+2*sin(psi)*sin(theta)*sin(phi);
    d2J_dtzdy_z = 2*cos(theta)*sin(phi);
    d2J_dphidy_z = 2*(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*((sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*y_z+(sin(psi)*cos(phi)-cos(psi)*sin(theta)*sin(phi))*z_z)+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))+2*(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*((-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*y_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*z_z)+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))+2*cos(theta)*sin(phi)*(cos(theta)*cos(phi)*y_z-cos(theta)*sin(phi)*z_z)+2*(-sin(theta)*x_z+cos(theta)*sin(phi)*y_z+cos(theta)*cos(phi)*z_z+tz-z_m)*cos(theta)*cos(phi);
    d2J_dthetady_z = 2*(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*(-cos(psi)*sin(theta)*x_z+cos(psi)*cos(theta)*sin(phi)*y_z+cos(psi)*cos(theta)*cos(phi)*z_z)+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*cos(psi)*cos(theta)*sin(phi)+2*(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*(-sin(psi)*sin(theta)*x_z+sin(psi)*cos(theta)*sin(phi)*y_z+sin(psi)*cos(theta)*cos(phi)*z_z)+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*sin(psi)*cos(theta)*sin(phi)+2*cos(theta)*sin(phi)*(-cos(theta)*x_z-sin(theta)*sin(phi)*y_z-sin(theta)*cos(phi)*z_z)-2*(-sin(theta)*x_z+cos(theta)*sin(phi)*y_z+cos(theta)*cos(phi)*z_z+tz-z_m)*sin(theta)*sin(phi);
    d2J_dpsidy_z = 2*(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*(-sin(psi)*cos(theta)*x_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*y_z+(cos(psi)*sin(phi)-sin(psi)*sin(theta)*cos(phi))*z_z)+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))+2*(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z)+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi));
    
    d2J_dposedy_z_ii = [ d2J_dtxdy_z ; d2J_dtydy_z ; d2J_dtzdy_z ; d2J_dphidy_z ; d2J_dthetady_z ; d2J_dpsidy_z ];

    d2J_dtxdz_z = 2*sin(psi)*sin(phi)+2*cos(psi)*sin(theta)*cos(phi);
    d2J_dtydz_z = -2*cos(psi)*sin(phi)+2*sin(psi)*sin(theta)*cos(phi);
    d2J_dtzdz_z = 2*cos(theta)*cos(phi);
    d2J_dphidz_z = 2*(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*((sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*y_z+(sin(psi)*cos(phi)-cos(psi)*sin(theta)*sin(phi))*z_z)+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*(sin(psi)*cos(phi)-cos(psi)*sin(theta)*sin(phi))+2*(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*((-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*y_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*z_z)+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))+2*cos(theta)*cos(phi)*(cos(theta)*cos(phi)*y_z-cos(theta)*sin(phi)*z_z)-2*(-sin(theta)*x_z+cos(theta)*sin(phi)*y_z+cos(theta)*cos(phi)*z_z+tz-z_m)*cos(theta)*sin(phi);
    d2J_dthetadz_z = 2*(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*(-cos(psi)*sin(theta)*x_z+cos(psi)*cos(theta)*sin(phi)*y_z+cos(psi)*cos(theta)*cos(phi)*z_z)+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*cos(psi)*cos(theta)*cos(phi)+2*(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*(-sin(psi)*sin(theta)*x_z+sin(psi)*cos(theta)*sin(phi)*y_z+sin(psi)*cos(theta)*cos(phi)*z_z)+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*sin(psi)*cos(theta)*cos(phi)+2*cos(theta)*cos(phi)*(-cos(theta)*x_z-sin(theta)*sin(phi)*y_z-sin(theta)*cos(phi)*z_z)-2*(-sin(theta)*x_z+cos(theta)*sin(phi)*y_z+cos(theta)*cos(phi)*z_z+tz-z_m)*sin(theta)*cos(phi);
    d2J_dpsidz_z = 2*(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*(-sin(psi)*cos(theta)*x_z+(-cos(psi)*cos(phi)-sin(psi)*sin(theta)*sin(phi))*y_z+(cos(psi)*sin(phi)-sin(psi)*sin(theta)*cos(phi))*z_z)+2*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z+tx-x_m)*(cos(psi)*sin(phi)-sin(psi)*sin(theta)*cos(phi))+2*(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*(cos(psi)*cos(theta)*x_z+(-sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi))*y_z+(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi))*z_z)+2*(sin(psi)*cos(theta)*x_z+(cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi))*y_z+(-cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi))*z_z+ty-y_m)*(sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi));
    
    d2J_dposedz_z_ii = [ d2J_dtxdz_z ; d2J_dtydz_z ; d2J_dtzdz_z ; d2J_dphidz_z ; d2J_dthetadz_z ; d2J_dpsidz_z ];

    d2J_dposedx_z(:,ii) = d2J_dposedx_z(:,ii) + d2J_dposedx_z_ii;
    d2J_dposedy_z(:,ii) = d2J_dposedy_z(:,ii) + d2J_dposedy_z_ii;
    d2J_dposedz_z(:,ii) = d2J_dposedz_z(:,ii) + d2J_dposedz_z_ii;

end
% fprintf('\n');
        
E2J_dposedz = [ d2J_dposedx_z  d2J_dposedy_z d2J_dposedz_z ];

Jacobian = inv(EHess)*E2J_dposedz;
CovZ = 0.05;
% CovX = inv(EHess)*E2J_dposedz * CovZ * E2J_dposedz'*inv(EHess);
CovX = Jacobian * CovZ * Jacobian';