function [CovM muV] = MeanSqError2(dataPair1,dataPair2,indexPoints)

%% Description:

% This function calculates the covariances errors matrix bthe idea is based

% on the convergence teorem from [BESL92], but calcualting correlation 

% between the (x,y,z) and (yaw,pitch,roll) angles 

%% Inputs :

% dataPair1 <- Reference Points

% datapair2 <- Model Points, the cloud to transform

% indexPoints <- index with correspondences between points

% L <- Value for the metric from Minguez et al

%% Outputs: 

% CovM -> Covariances matrix (6x6)

% muV -> root mean square's Vector (1x6)

% ek2 -> 

%% Abreviations

% e : error

% sig : sigma



out = nargout;



if out == 0

    error('Not enough output arguments')

elseif out >= 4

    error('Too many output arguments')

end



eX2 = 0;

eY2 = 0;

eZ2 = 0;

ek2 = 0;

eYaw2 = 0;

ePit2 = 0;

eRol2 = 0;

Np = size(indexPoints,1); 

x = 1;

y = 2;

z = 3;

%% Computing the Mean Square Errors

for i = 1:Np

%     fprintf('MSE i : %d \n', i)

    i1 = indexPoints(i,1);

    i2 = indexPoints(i,2);

    % Computing the (x,y,z) square errors

    delta_i(i,1:3) = dataPair1(i1,1:3)-dataPair2(i2,1:3);

%     fprintf('Delta_i: %d %d \n',[i1 i2]);

    ek2 = ek2 + norm(delta_i)^2;

    eX2 = eX2 + delta_i(1,x)^2;

    eY2 = eY2 + delta_i(1,y)^2;

    eZ2 = eZ2 + delta_i(1,z)^2;

    % Computing the yaw,pitch and roll square errors.

    eRol2 = eRol2 + (atan(dataPair1(i1,z)/dataPair1(i1,y))-atan(dataPair2(i2,z)/dataPair2(i2,y)))^2;

    ePit2 = ePit2 + (atan(dataPair1(i1,x)/dataPair1(i1,z))-atan(dataPair2(i2,x)/dataPair2(i2,z)))^2;

    eYaw2 = eYaw2 + (atan(dataPair1(i1,y)/dataPair1(i1,x))-atan(dataPair2(i2,y)/dataPair2(i2,x)))^2;

    end

end

% Computing the mean square errors

 ek2= ek2/Np;

if out  > 1 

    eX2 = eX2/Np;

    eY2 = eY2/Np;

    eZ2 = eZ2/Np;

    eRol2 = eRol2/Np;

    ePit2 = ePit2/Np;

    eYaw2 = eYaw2/Np;

end



%% Computing the Mean

if out  > 1 

    sigX = 0;

    sigY = 0;

    sigZ = 0;

    sigRol = 0;

    sigPit = 0;

    sigYaw = 0;

    muX = sqrt(eX2);

    muY = sqrt(eY2);

    muZ = sqrt(eZ2);

    muRol = sqrt(eRol2);

    muPit = sqrt(ePit2);

    muYaw = sqrt(eYaw2);

   

    for i = 1:Np

        i1 = indexPoints(i,1);

        i2 = indexPoints(i,2);

        mu1 = mean(dataPair1(1:3));

        mu2 = mean(dataPair2(1:3));

        % Computing the (x,y,z) errors covariance

        delta_i = dataPair1(i1,1:3)-dataPair2(i2,1:3);

        sigX = sigX + (delta_i(1,x)-muX);

        sigY = sigY + (delta_i(1,y)-muY);

        sigZ = sigZ + (delta_i(1,z)-muY);

        % Computing the yaw,pitch and roll errors.

        sigRol = sigRol + (atan(dataPair1(i1,z)/dataPair1(i1,y))-atan(dataPair2(i2,z)/dataPair2(i2,y)))-muRol;

        sigPit = sigPit + (atan(dataPair1(i1,x)/dataPair1(i1,z))-atan(dataPair2(i2,x)/dataPair2(i2,z)))-muPit;

        sigYaw = sigYaw + (atan(dataPair1(i1,y)/dataPair1(i1,x))-atan(dataPair2(i2,y)/dataPair2(i2,x)))-muYaw;    

    end

%% root mean square's Vector 

     muV = [ muX; muY; muZ; muRol; muPit; muYaw ];

     

%% Covariances matrix



     CovM = [sigX^2      sigX*sigY    sigX*sigZ   sigX*sigRol   sigX*sigPit   sigX*sigYaw

            sigX*sigY  sigY^2       sigY*sigZ   sigY*sigRol   sigY*sigPit   sigY*sigYaw

            sigX*sigZ   sigY*sigZ    sigZ^2      sigZ*sigRol   sigZ*sigPit   sigZ*sigYaw

            sigX*sigRol sigY*sigRol  sigZ*sigRol sigRol^2      sigRol*sigPit sigRol*sigYaw

            sigX*sigPit sigY*sigPit  sigZ*sigPit sigRol*sigPit sigPit^2      sigPit*sigYaw

            sigX*sigYaw sigY*sigYaw  sigZ*sigYaw sigYaw*sigRol sigPit*sigYaw sigYaw^2 ];

    CovM = CovM/(Np-1);

end

return

