% ERROR2
% Given two data set with the same size .Calculates mean square error 
% based on [BESL92], 
%
%% Description
%
% Author - 	
%           Ernesto Homar Teniente Aviles
%       CONACYT Scholar
%       Institut de Robotica i Informatica Industrial, CSIC-UPC
%       Last change: 07/2010
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
%
% Usage
%   err2 = error2(Xr,Xq)
%
% Variables:
%   Inputs 
%       Xr   - 3xn matrix, reference Points Xr
%       Xq   - 3xn matrix, query(model) Points, Xq
% Outputs: 
%       err2  - a integer, mean square error
%


%% Code
function err2 = error2(Xr,Xq,varargin)

[n1 m1] = size(Xr);
[n2 m2] = size(Xq);

if m1 ~= m2
    error('Mismatching number of elements betwen Xr and Xq ')
end
if n1 ~= 3 || n2 ~= 3
    error('Not a valid dimension for Xr or Xq ')
end

[L  ...
 type] = process_options(varargin, ...
                         'L', 30,  ...
                         'Type', 'Mb');


% We are computing Mean error instead MSE
switch type
  case 'Mb'
%     err2 = sum(MbDist(Xq,Xr,L))/size(Xr,2);
    err2 = sum(MbDistc(Xq,Xr,L))/size(Xr,2);
  case {'SVD', 'Quat'}
   err2 = sum(dist3D(Xr,Xq))/size(Xr,2)^2;
end

