%% errorCov
% Find the covariances matrix and mu vector for the associated points

%% Description

% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 5/2008
% 
% Usage
%       [CovM muV] = errorCov(data1,data2,inxPoints)
%
% Varibles:
%   Inputs
%       data1  - a 3 x n matrix, Reference Points
%       data2  - a 3 x n matrix, Model Points, the cloud to transform
% 
%   Outputs:
%       CovM    - a 6 x 6 matrix, Covariances matrix (6x6)
%       muV     - a 1 x 6 vector, median component error 
%                         (x,y,z,roll,pitch,yaw)

%% Code:
function [CovM muV] = errorCov(data1,data2)

[r1 c1] = size(data1);
[r2 c2] = size(data2);

if r1~=r2
    error('Invalid dimension compatibility between data and query points')
end

if (c1 < 3 || c2 < 3)
   error('Mismatching dimentions for the query or reference data')
end


%%
x = 1; y = 2; z = 3; rx = 4; ry = 5; rz = 6;

err = zeros(r1,6);
err2 = zeros(r1,6);

muErr = zeros(6,1);
muErr2 = zeros(6,1);

% figure(figNo);
% hold on; axis equal; grid on;
%%
for i=1:r1
    % Computing the mean for (x,y,z) yaw, pitch and roll
    err(i,1:3) = abs(data1(i,1:3)-data2(i,1:3));
    err(i,4) = abs(atan(data1(i,z)/data1(i,y))-atan(data2(i,z)/...
                    data2(i,y)));
    err(i,5) = abs(atan(data1(i,x)/data1(i,z))-atan(data2(i,x)/...
                    data2(i,z)));
    err(i,6) = abs(atan(data1(i,y)/data1(i,x))-atan(data2(i,y)/...
                    data2(i,x)));
    
    err2(i,x) = err(i,1)^2;
    err2(i,y) = err(i,2)^2;% (data1(i,y)-data2(i,y))^2;
    err2(i,z) = err(i,3)^2;% (data1(i,z)-data2(i,z))^2;
    err2(i,rx) = err(i,4)^2;
    err2(i,ry) = err(i,5)^2;
    err2(i,rz) = err(i,6)^2;
    muErr2(x,1) = muErr2(x,1) + err2(i,x);
    muErr2(y,1) = muErr2(y,1) + err2(i,y);
    muErr2(z,1) = muErr2(z,1) + err2(i,z);
    muErr2(rx,1) = muErr2(rx,1) + err2(i,rx);
    muErr2(ry,1) = muErr2(ry,1) + err2(i,ry);
    muErr2(rz,1) = muErr2(rz,1) + err2(i,rz);
end
%%
% Computing the median values (mean squared error)
muErr2 = muErr2/r1;
muV(:,1) = muErr2;
%errx*
errxx = 0; errxy = 0; errxz = 0; errxrol = 0; errxpitch = 0;errxyaw = 0;
% erry*
erryy = 0; erryz = 0; erryrol = 0; errypitch = 0; erryyaw = 0; 
% errz*
errzz = 0; errzrol = 0; errzpitch = 0; errzyaw = 0;
% errrol*
errrolrol = 0; errrolpitch = 0; errrolyaw = 0;
% errpitch*
errpitchpitch = 0; errpitchyaw = 0;
% erryaw*
erryawyaw = 0;
for i=1:r1
    % errx
    errxx = errxx+err(i,x)*err(i,x);
    errxy = errxy+err(i,x)*err(i,y);
    errxz = errxz+err(i,x)*err(i,z);
    errxrol = errxrol + err(i,x)*err(i,rx);
    errxpitch = errxpitch + err(i,x)*err(i,ry);
    errxyaw = errxyaw +err(i,x)*err(i,rz);
    
    % erry
    erryy = erryy + err(i,y)*err(i,y);
    erryz = erryz + err(i,y)*err(i,z);
    erryrol = erryrol + err(i,y)*err(i,rx);
    errypitch = errypitch + err(i,y)*err(i,ry);
    erryyaw = erryyaw + err(i,y)*err(i,rz);
    
    % errz
    errzz = errzz + err(i,z)*err(i,z);
    errzrol = errzrol + err(i,z)*err(i,rx);
    errzpitch = errzpitch + err(i,z)*err(i,ry);
    errzyaw = errzyaw + err(i,z)*err(i,rz);
    
    % errrol
    errrolrol = errrolrol + err(i,rx)*err(i,rx);
    errrolpitch = errrolpitch + err(i,rx)*err(i,ry);
    errrolyaw = errrolyaw + err(i,rx)*err(i,rz);
    
    % errpitch
    errpitchpitch = errpitchpitch + err(i,ry)*err(i,ry);
    errpitchyaw = errpitchyaw + err(i,ry)*err(i,rz);
    
    % erryaw    
    erryawyaw = erryawyaw + err(i,rz)*err(i,rz);


end

    % errx
    errxx = errxx/r1;
    errxy = errxy/r1;
    errxz = errxz/r1;
    errxrol = errxrol/r1;
    errxpitch = errxpitch/r1;
    errxyaw = errxyaw/r1;
    
    % erry
    erryy = erryy/r1;
    erryz = erryz/r1;
    erryrol = erryrol/r1;
    errypitch = errypitch/r1;
    erryyaw = erryyaw/r1;
    
    % errz
    errzz = errzz/r1;
    errzrol = errzrol/r1;
    errzpitch = errzpitch/r1;
    errzyaw = errzyaw/r1;
    
    % errrol
    errrolrol = errrolrol/r1;
    errrolpitch = errrolpitch/r1;
    errrolyaw = errrolyaw/r1;
    
    % errpitch
    errpitchpitch = errpitchpitch/r1;
    errpitchyaw = errpitchyaw/r1;
    
    % erryaw    
    erryawyaw = erryawyaw/r1;
%%    
CovM =[ errxx       errxy       errxz       errxrol     errxpitch       errxyaw
        errxy       erryy       erryz       erryrol     errypitch       erryyaw
        errxz       erryz       errzz       errzrol     errzpitch       errzyaw
        errxrol     erryrol     errzrol     errrolrol   errrolpitch     errrolyaw
        errxpitch   errypitch   errzpitch   errrolpitch errpitchpitch  errpitchyaw
        errxyaw     erryyaw     errzyaw     errrolyaw   errpitchyaw     erryawyaw];
    
return