% ERRORCOV2 - Find the covariances matrix and mu vector for the associated points
%
%% Description
%
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 3/2009
% 
% Usage
%       [CovM muV] = errorCov2(data1,data2)
%
% Varibles:
%   Inputs
%       data1  - a 3 x n matrix, Reference Points
%       data2  - a 3 x n matrix, Model Points, the cloud to transform
% 
%   Outputs:
%       CovM    - a 6 x 6 matrix, Covariances matrix (6x6)
%       muV     - a 1 x 6 vector, median component error 
%                         (x,y,z,roll,pitch,yaw)

%% Code:
function covM = errorCov2(data1,data2)

[r1 c1] = size(data1);
[r2 c2] = size(data2);

if r1~=r2
    error('Invalid dimension compatibility between data and query points')
end

if (c1 < 3 || c2 < 3)
   error('Mismatching dimentions for the query or reference data')
end


%%

x = 1; y = 2; z = 3;
rx = 4; ry = 5; rz = 6;

err = zeros(r1,6);

% figure(figNo);
% hold on; axis equal; grid on;
%%
for i=1:r1
    % Computing the mean for (x,y,z) yaw, pitch and roll
    err(i,x:z) = abs(data1(i,1:3)-data2(i,1:3));

    err(i,rx) = abs(atan(data1(i,z)/data1(i,y))-atan(data2(i,z)/...
                    data2(i,y)));
    err(i,ry) = abs(atan(data1(i,x)/data1(i,z))-atan(data2(i,x)/...
                    data2(i,z)));
    err(i,rz) = abs(atan(data1(i,y)/data1(i,x))-atan(data2(i,y)/...
                    data2(i,x)));
end
%%

covM = (err'*err)/r1;

% covM= cov(err);



