% NAIVECOVARINCE
% Find the covariances matrix for the matched points
%
%% Description
%
% Author - 	
%       Ernesto Homar Teniente Aviles
% 			CONACYT Schoolar
%       Institut de Robotica i Informatica Industrial, CSIC-UPC
%       Last change: 11/2010
% 
% Usage
%       [CovM muV] = errorCov(data1,data2)
%
% Varibles:
%   Inputs
%       data1  - a 3 x n matrix, Reference Points
%       data2  - a 3 x n matrix, Model Points, the cloud to transform
% 
%   Outputs:
%       covM    - a 6 x 6 matrix, Covariances matrix (6x6)
%                         (x,y,z,_rxl,_ry,_rz)

%% Code:
function covM  = naiveCovariance(data1,data2)

[r1 c1] = size(data1);
[r2 c2] = size(data2);

if r1~=r2
    error('Invalid dimension compatibility between data and query points')
end

if (c1 < 3 || c2 < 3)
   error('Mismatching dimentions for the query or reference data')
end


%%
x = 1; y = 2; z = 3;

err.t = abs(data1-data2); 
data.x1 = data1(x,:);
data.y1 = data1(y,:);
data.z1 = data1(z,:);
data.x2 = data2(x,:);
data.y2 = data2(y,:);
data.z2 = data2(z,:);

err.rx = abs(atan(data.z1./data.y1)-atan(data.z2./data.y2));
err.ry = abs(atan(data.x1./data.z1)-atan(data.x2./data.z2));
err.rz = abs(atan(data.y1./data.x1)-atan(data.y2./data.x2));

% 
err.xx = sum(err.t(x,:).*err.t(x,:));
err.xy = sum(err.t(x,:).*err.t(y,:));
err.xz = sum(err.t(x,:).*err.t(z,:));
err.xrx = sum(err.t(x,:).*err.rx);
err.xry = sum(err.t(x,:).*err.ry);
err.xrz = sum(err.t(x,:).*err.rz);


err.yy = sum(err.t(y,:).*err.t(y,:));
err.yz = sum(err.t(y,:).*err.t(z,:));
err.yrx = sum(err.t(y,:).*err.rx);
err.yry = sum(err.t(y,:).*err.ry);
err.yrz = sum(err.t(y,:).*err.rz);

err.zz = sum(err.t(z,:).*err.t(z,:));
err.zrx = sum(err.t(z,:).*err.rx);
err.zry = sum(err.t(z,:).*err.ry);
err.zrz = sum(err.t(z,:).*err.rz);

err.rxrx = sum(err.rx.*err.rx);
err.rxry = sum(err.rx.*err.ry);
err.rxrz = sum(err.rx.*err.rz);

err.ryry = sum(err.ry.*err.ry);
err.ryrz = sum(err.ry.*err.rz);

err.rzrz = sum(err.rz.*err.rz);


covM =[ err.xx  err.xy  err.xz  err.xrx  err.xry   err.xrz
        err.xy  err.yy  err.yz  err.yrx  err.yry   err.yrz
        err.xz  err.yz  err.zz  err.zrx  err.zry   err.zrz
        err.xrx err.yrx err.zrx err.rxrx err.rxry  err.rxrz
        err.xry err.yry err.zry err.rxry err.ryry  err.ryrz
        err.xrz err.yrz err.zrz err.rxrz err.ryrz  err.rzrz]/c1;

      
      
% err = zeros(3,c1); 
% 
% %errx*
% errxx = 0; errxy = 0; errxz = 0; errx_rx = 0; errx_ry = 0;errx_rz = 0;
% % erry*
% erryy = 0; erryz = 0; erry_rx = 0; erry_ry = 0; erry_rz = 0; 
% % errz*
% errzz = 0; errz_rx = 0; errz_ry = 0; errz_rz = 0;
% % err_rx*
% err_rx_rx = 0; err_rx_ry = 0; err_rx_rz = 0;
% % err_ry*
% err_ry_ry = 0; err_ry_rz = 0;
% % err_rz*
% err_rz_rz = 0;
      

% for i=1:c1
%   err = err.t(:,i);
%   err_rx = abs(atan(data1(z,i)/data1(y,i))-atan(data2(z,i)/...
%                   data2(y,i)));
%   err_ry = abs(atan(data1(x,i)/data1(z,i))-atan(data2(x,i)/...
%                   data2(z,i)));
%   err_rz = abs(atan(data1(y,i)/data1(x,i))-atan(data2(y,i)/...
%                   data2(x,i)));
% 
%   errxx = errxx+err(x)*err(x);
%   errxy = errxy+err(x)*err(y);
%   errxz = errxz+err(x)*err(z);
%   errx_rx = errx_rx + err(x)*err_rx;
%   errx_ry = errx_ry + err(x)*err_ry;
%   errx_rz = errx_rz +err(x)*err_rz;
%   % erry
%   erryy = erryy + err(y)*err(y);
%   erryz = erryz + err(y)*err(z);
%   erry_rx = erry_rx + err(y)*err_rx;
%   erry_ry = erry_ry + err(y)*err_ry;
%   erry_rz = erry_rz + err(y)*err_rz;
%   % errz
%   errzz = errzz + err(z)*err(z);
%   errz_rx = errz_rx + err(z)*err_rx;
%   errz_ry = errz_ry + err(z)*err_ry;
%   errz_rz = errz_rz + err(z)*err_rz;
%   % err_rx
%   err_rx_rx = err_rx_rx + err_rx*err_rx;
%   err_rx_ry = err_rx_ry + err_rx*err_ry;
%   err_rx_rz = err_rx_rz + err_rx*err_rz;
%   % err_ry
%   err_ry_ry = err_ry_ry + err_ry*err_ry;
%   err_ry_rz = err_ry_rz + err_ry*err_rz;
%   % err_rz    
%   err_rz_rz = err_rz_rz + err_rz*err_rz;
%                   
% end
% %%    
% covM =[ errxx   errxy   errxz   errx_rx   errx_ry    errx_rz
%         errxy   erryy   erryz   erry_rx   erry_ry    erry_rz
%         errxz   erryz   errzz   errz_rx   errz_ry    errz_rz
%         errx_rx erry_rx errz_rx err_rx_rx err_rx_ry  err_rx_rz
%         errx_ry erry_ry errz_ry err_rx_ry err_ry_ry  err_ry_rz
%         errx_rz erry_rz errz_rz err_rx_rz err_ry_rz  err_rz_rz]/r1;
    
