% BESLICP3DQUATMINM -
% Compute the translation a unit quaternion vector wich minimize th rigid body
% transformation between two data sets according with the minimizations process
% proposed by Horn_88
%
%% Description:
%
% Author - 	
%       Ernesto Homar Teniente Aviles
% 			CONACYT schoolar
%       Institut de Robotica i Informatica Industrial, CSIC-UPC
%       Last change: 07/2010
%   		Version 1.1
%
% Usage:
%   HRt =  BeslICP3DquatMinm(refData,newData)
%
% Variables:
%   Inputs:
%
% 	Outputs:
%			HRt	- 1 x 3 vector, traslation
%
%
%	Globals
%		refData	- a n x 3 matrix, first correspondences  data set 
%		newData	- a n x 3 matrix, second correspondences data set (lastest)
%
% Notes:
% Formulas are from in nuchter_book08, pages 42-45
%
% some similarities to the variables are:
%
% refData = z_t (Censi)= S_ref (Biota) = p_2 = M (Nüchter) = B (Aleksandr) = X (Besl)
% newData = z_t+1 (Censi) = S_new (Biota) = p_1 = D (Nüchter)= A (Aleksandr) = P (Besl)

%% Code
function HRt =  beslICP3DquatMinm(refData,newData)


[Mp c_m]= getSetPointPrima(refData);
[Dp c_d]= getSetPointPrima(newData);

% Correlation Matrix
H = getCorrMatrix3D(Dp,Mp);

% Minimum quaternion
q_min  = qMinm(H);

% Rotation matrix
R = q2R(q_min);
t = c_m - rotateData3D(c_d,R);
HRt = [ R t ; zeros(1,3) 1];
