% BESLICP3DSVDMINIM -
% Compute the translation a unit quaternion vector wich minimize th rigid body
% transformation between two data sets according with the minimizations process
% proposed by Arun_pami87
%
%% Description:
%
% Author - 	
%       Ernesto Homar Teniente Aviles
% 			CONACYT Schoolar
%       Institut de Robotica i Informatica Industrial, CSIC-UPC
%       Last change: 07/2010
%       Version 1.1
%
% Usage:
%       HRt = beslICP3DsvdMinm(refDta, newData)
%
% Variables:
%   Inputs:
%
%
% 	Outputs:
%			HRt	- 4x4 matrix, homogeneus trasformation matrix
%
%
%	Globals
%		refData	- a n x 3 matrix, first correspondences  data set 
%		newData	- a n x 3 matrix, second correspondences data set (lastest)
%
% Notes:
% Formulas are from in nuchter_book08, pages 42-45, based on the method 
% inventend .
% some similarities to the 
% variables are:
%
% dataP = z_t = S_ref = P_1 = m = refData
% newData = z_t+1 = S_new = P_2 = d = newData

%% Code:

function HRt = beslICP3DsvdMinm(refData,newData)


[Mp c_m]= getSetPointPrima(refData);
[Dp c_d]= getSetPointPrima(newData);

% Correlation Matrix

H = getCorrMatrix3D(Dp,Mp);
 
[U Sa V] = svd(H);

% Rotation matrix
R = V*U';
% Translation
t = c_m - rotateData3D(c_d,R);

HRt = [ R t ; zeros(1,3) 1];
