%%  MbICP3DdJdq. Compute the matrix A for the metric-based ICP 3D Biota_icra06

%% Description:
%
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 07/2009
%			Version 1.0
%
% Usage
%
% Variables:
%   Inputs
%
% Outputs:
%
% Notes:
% Formulas are written as stated in Biotacra06, some similarities to the 
% variables are:
%
% dataP = z_t = S_ref = P_1
% dataPp = z_t+1 = S_new = P_2
%
% If you have trobles with your results try inverting the data when in you 
% define dataPp, it is:
%
% dataPp = z_t = S_ref = P_1
% dataP = z_t+1 = S_new = P_2

%% Code
function dJ = MbICP3DdJdq(A,b)

[an am] = size(A);
bn = size(b,1);
if (an~= 6) || (an~= 6) || (bn~= 6)
	error('Mismatch dimentions ofr matrix A or vector b.');
end

global q;

x = q(1,1);
y = q(2,1);
z = q(3,1);
thnx = q(4,1);
thny = q(5,1);
thnz = q(6,1);


dJ.dx = 2*(A(1,1)*x+A(1,2)*y+A(1,3)*z+A(1,4)*thnx+A(1,5)*thny+A(1,6)*thnz+b(1,1);
dJ.dy = 2*(A(2,1)*x+A(2,2)*y+A(2,3)*z+A(2,4)*thnx+A(2,5)*thny+A(2,6)*thnz+b(2,1);
dJ.dz = 2*(A(3,1)*x+A(3,2)*y+A(3,3)*z+A(3,4)*thnx+A(3,5)*thny+A(3,6)*thnz+b(3,1);
dJ.thnx = 2*(A(4,1)*x+A(4,2)*y+A(4,3)*z+A(4,4)*thnx+A(4,5)*thny+A(4,6)*thnz+b(4,1);
dJ.thny = 2*(A(5,1)*x+A(5,2)*y+A(5,3)*z+A(5,4)*thnx+A(5,5)*thny+A(5,6)*thnz+b(5,1);
dJ.thnz = 2*(A(6,1)*x+A(6,2)*y+A(6,3)*z+A(6,4)*thnx+A(6,5)*thny+A(6,6)*thnz+b(6,1);


