%%  MbICP3Dcovz. Conmpute the covariance for the metric-based ICP 3D from
%% z = (z_t-1,z_t)
%
%% Description:
%
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 07/2009
%			Version 1.0
%
% Usage
%		cov_q= MbICP3Dcovz(A)
% Variables:
%   Inputs:
%		A - a 6x6 matrix, computed from the minimization function
%		cov_z - a (nx6)x6 matrix, covariances from z
%		cov_z = [cov_zt_1	 0	  	0		...		0		0		... 	0
%					0	 cov_zt_2 	0		...		0		0		... 	0	
%					.	 	 .		.		.		.		.		.		.
%					.	 	 .		.		 .		.		.		 .		.
%					.	 	 .		.		  .		.		.		  .		.
%					0	 	0    cov_zt_n	...		0		0		... 	0
%					0		0		0		...	cov_zt1_1	0		...		0
%					0		0		0		...		0	cov_zt1_2 	...		0		
%					.		.		.		.		.	 	.		.		.	
%					.		.		.		 .		.	 	.		 .		.	
%					.		.		.		  .		.	 	.		  .		.	
%					0		0		0		...		0	 	0		... cov_zt1_n
%					]
%
% Outputs:
%		coz_q - a 6x6 matrix, covariance matrix computed from q and z
%
% Notes:
% Formulas are written as stated in Biotacra06, some similarities to the 
% variables are:
%
% dataP = z_t-1 = S_ref = P_1
% dataPp = z_t = S_new = P_2
%
% If you have trobles with your results try inverting the data when in you 
% define dataPp, it is:
%
% dataPp = z_t-1 = S_ref = P_1
% dataP = z_t = S_new = P_2


%% Code
function cov_q= MbICP3Dcovz(A,cov_z)

global dataP dataPp q L;

d2Jdzdq = MbICP3Dd2Jdzdq;
invd2Jdq = inv(MBicp3Dd2Jdq(A));
% crear solo una funcion covMMA. Covariance matrix of a minimimization
% algorithm
cov_q = invd2Jdq*d2Jdzdq*cov_z*d2Jdzdq'*invd2Jdq
