% MBICP3DCOVZT
% Cnmpute closed form metric-based ICP covariance from z_t
%
%% Description:
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 07/2009
%			Version 1.0
%
% Usage:
%   cov_q= MbICP3Dcovzt(A)
%
% Variables:
%   Inputs
%		A - a 6x6 matrix, computed from the minimization function
%
% Outputs:
%		coz_q - a 6x6 matrix, covariance matrix computed from q and z_t
%
% Global:
%   cov_zt 
%   dataP
%   dataPp
%   q
%   L
%
%
% Notes:
% Formulas are written as stated in Biotacra06, some similarities to the 
% variables are:
%
% dataP = z_t-1 = S_ref = P_1
% dataPp = z_t = S_new = P_2
%
% If you have trobles with your results try inverting the data when in you 
% define dataPp, it is:
%
% dataPp = z_t-1 = S_ref = P_1
% dataP = z_t = S_new = P_2

%% Code:
function cov_q= MbICP3Dcovzt(A,b)

global cov_zt;

% dataP= c.s;
% dataPp= c.m;


d2Jdztdq = MbICP3Dd2Jdztdq(A,b);
invd2Jdq = inv(MbICP3Dd2Jdq(A));
% crear solo una funcion covMMA. Covariance matrix of a minimimization
% algorithm
cov_q = invd2Jdq*d2Jdztdq*cov_zt*d2Jdztdq'*invd2Jdq;
