% MBICP3DD2JDZTDQ -
% Compute the second derivatives with respect q and  z_t
%
%% Description:
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 07/2009
%			Version 1.0
%
% Usage:
%       d2Jdztdq = MbICP3Dd2Jdztdq(A,b) 
% 
% Variables:
%   Inputs
%
% Outputs:
%		d2Jdzdq - a 6x(3xn) matrix, n number of correspondences
%
% Notes:
% Some similarities to the variables are:
%
% dataP = z_t-1 = S_ref = P_1
% dataPp = z_t = S_new = P_2
%
% If you have trobles with your results try inverting the data when in you 
% define dataPp, it is:
%
% dataPp = z_t-1 = S_ref = P_1
% dataP = z_t = S_new = P_2


%% Code:
function d2Jdztdq = MbICP3Dd2Jdztdq(A,b) 

% [an am] = size(A);
% bn = size(b,1);
% if (an~= 6) || (an~= 6) || (bn~= 6)
% 	error('Mismatch dimentions ofr matrix A or vector b.');
% end

global dataP dataPp q L;

%z_t1 = dataP;
%z_t = dataPp;
%px = dataP(:,1);
%py = dataP(:,2);
%pz = dataP(:,3);
%pxp = dataPp(:,1);
%pyp = dataPp(:,2);
%pzp = dataPp(:,3);

x = q(1,1);
y = q(2,1);
z = q(3,1);
thnx = q(4,1);
thny = q(5,1);
thnz = q(6,1);

n = size(dataPp,1);

%% dJ = MbICP3DdJdq(A,b);

d2Jdztdqtmp = zeros(6,3*n);

for  i=1:n
	
	% k = norm(dataP(i,1:3))^2 + L^2;
	pix= dataP(i,1);
	piy= dataP(i,2);
	piz= dataP(i,3);
	pixp= dataPp(i,1);
	piyp= dataPp(i,2);
	pizp= dataPp(i,3);
	
%% dztidx			
	d2J.dpixpdx = 2*(pix^2+L^2-thny*pix*piz+thnz*pix*piy);
	d2J.dpiypdx = 2*(thnx*pix*piz-thnz*(pix^2+L^2)+pix*piy);
 	d2J.dpizpdx = 2*(-2*thnx*pix*piy+thny*(pix^2+L^2)+pix*piz);
 	
 	d2J.dztidx = [d2J.dpixpdx d2J.dpiypdx d2J.dpizpdx];
%% dztidy

	d2J.dpixpdy = 2*(-thny*piy*piz+thnz*(piy^2+L^2)+pix*piy);
 	d2J.dpiypdy = 2*(thnx*piy*piz-thnz*pix*piy+piy^2+L^2);
 	d2J.dpizpdy = 2*(-thnx*(piy^2+thnx*L^2)+thny*pix*piy+piy*piz);
 	
 	d2J.dztidy = [d2J.dpixpdy d2J.dpiypdy d2J.dpizpdy];
%% dztidz
	d2J.dpixpdz = 2*(-thny*(piz^2+L^2)+thnz*piy*piz+pix*piz);
	d2J.dpiypdz = 2*(thnx*(piz^2+L^2)-thnz*pix*piz+piy*piz);
	d2J.dpizpdz = 2*(-thnx*piy*piz+thny*pix*piz+piz^2+L^2);
	
	d2J.dztidz = [d2J.dpixpdz d2J.dpiypdz d2J.dpizpdz];
%% dztidthnx	
	d2J.dpixpdthnx = 2*(thny*(pizp*piy*piz-piyp*piz^2-piyp*L^2)+...
		thnz*(piyp*piy*piz-pizp*piy^2-pizp*L^2)+pix*(piy*pizp-piz*piyp));
 	d2J.dpiypdthnx = 2*(x*pix*piz+y*piy*piz+z*(piz^2+L^2)+...
		thnz*(2*piyp*L^2+2*piyp*piz^2-2*piy*piz*pizp)+...
		thny*(pix*piz*pizp-pixp*piz^2-pixp*L^2)+...
		thnz*(pizp*pix*piy+piy*piz*pixp-2*pix*piz*piyp)+...
		pix*(pix*piz-piz*pixp)+piy*(pizp*piy-2*piz*piyp+piy*piz)+...
		piz*(L^2+piz^3-piz*pizp));
 	d2J.dpizpdthnx = 2*(-x*pix*piy-y*(piy^2+L^2)-z*piy*piz+...
	+2*thnx*(pizp*piy^2-pizp*L^2-piy*piz*piyp)+...
	thny*(piy*piz*pixp+piyp*pix*piz-2*pix*piy*pizp)+...
	thnz*(pix*piy*piyp-pixp*piy^2-pixp*L^2)+pix*piy*(pixp-pix)+...
	piy*(-2*piy^2-L^2+piy*piyp)+piy*piz*(2*pizp-piz)+piyp*piz^2);
	
	d2J.dztidthnx = [d2J.dpixpdthnx d2J.dpiypdthnx d2J.dpizpdthnx];
%% dztidthny
	d2J.dpixpdthny = 2*(-x*pix*piz-y*piy*piz-z*(piz^2+L^2)+...
		thnx*(piy*piz*pizp-piyp*piz^2-piyp*L^2)+...
		2*thny*(pixp*piz^2+pixp*L^2-pix*piz*pizp)+...
		thnz*(pix*piz*piyp-2*piy*piz*pixp+pizp*pix*piy)+...
        pix*(2*piz*pixp-pix*pizp)+...
		piz*(piy*piyp+piz*pizp-pix^2-piy^2-piz^2-L^2));
 	d2J.dpiypdthny = 2*(thnx*(pix*piz*pizp-pixp*piz^2-pixp*L^2)+...
		thnz*(pixp*pix*piz-pizp*pix^2-pizp*L^2)+piy*(piz*pixp-pix*pizp));
 	d2J.dpizpdthny = 2*(x*(pix^2+L^2)+z*pix*piz+y*pix*piy+...
		thnz*(pix*piy*pixp-piyp*pix^2-piyp*L^2)+...
		thnx*(piy*piz*pixp+pix*piz*piyp-2*pix*piy*pizp)+...
	 	2*thny*(pizp*pix^2+pizp*L^2-pix*piz*pixp)+...
	 	pix*(-2*piz*pizp-piy*piyp-pix*pixp+pix^2+piy^2+piz^2+L^2)+pixp*piz^2);


	d2J.dztidthny = [d2J.dpixpdthny d2J.dpiypdthny d2J.dpizpdthny];
%% dztidthnz	
	d2J.dpixpdthnz = 2*(x*pix*piy+z*piy*piz+y*(piy^2+L^2)+...
		thnx*(piy*piz*piyp-pizp*L^2-pizp*piy^2)+...
		thny*(pix*piy*pizp+piyp*pix*piz-2*piy*piz*pixp)+...
		2*thnz*(pixp*piy^2+pixp*L^2-pix*piy*piyp)+pix^2*piyp+...
		piy*(-2*pix*pixp-piz*pizp-piy*piyp+pix^2+piy^2+piz^2+L^2));
	d2J.dpiypdthnz = 2*(-x*(pix^2+L^2)-y*pix*piy-z*pix*pizp+...
		thnx*(pix*piy*pizp-2*pix*piz*piyp+piy*piz*pix)+...
		thny*(pix*piz*pixp-pizp*pix^2-pizp*L^2)+...
		2*thnz*(piyp*L^2+piyp*pix^2-pix*piy*pixp)-piy^2*pixp+...
		pix*(piz*pizp+2*piy*piyp+pix*pixp-pix^2-piy^2-piz^2-L^2));
	d2J.dpizpdthnz = 2*(thnx*(pix*piy*piyp-pixp*piy^2-pixp*L^2)+...
		thny*(pix*piy*pixp-piyp*pix^2-piyp*L^2)+piz*(pix*piyp-piy*pixp));

	d2J.dztidthnz = [d2J.dpixpdthnz d2J.dpiypdthnz d2J.dpizpdthnz];
 				
	d2Jdztdqtmp(:,3*(i-1)+1:3*i) = [d2J.dztidx; d2J.dztidy; d2J.dztidz
							d2J.dztidthnx; d2J.dztidthny; d2J.dztidthnz];
end

d2Jdztdq =d2Jdztdqtmp;

