%%  MBICPGETAB -
% Compute A matrix and b vector for the metric-based ICP 3D Biota_icra06
%
%% Description:
%
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Scholarship
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 07/2009
%			Version 1.0
%
% Usage
%
% Variables:
%   Inputs
%
%   Outputs:
%       A - 6x6 matrix
%       b - 6 x1 vector
%
%   Globals:
%   dataP  - nx3
%   dataPp - nx3 
%   L      - an integer
%
% Notes:
% Formulas are written as stated in Biotacra06, some similarities to the 
% variables are:
%
%  dataP = z_t-1 c.m (Censi),S_ref p_2 (Biota),M (Nüchter),B (Aleksandr) 
%       , X (Besl)
%  dataPp = z_t c.s (Censi),S_new p_1 (Biota),D (Nüchter), A (Aleksandr)
%       , P (Besl)
%
% If you have trobles with your results try inverting the data when you 
% define dataPp

%% Code
function [A b] = MbICP3DgetAb(dataP, dataPp, L)

% global dataP dataPp L;

n1 = size(dataP,1);

Atmp = zeros(6,6);
btmp = zeros(6,1);

for i = 1:n1
    
    %k = norm(dataP(i,1:3))^2 + L^2;
    M =[dataP(i,1)^2+L^2        dataP(i,1)*dataP(i,2)   dataP(i,1)*dataP(i,3) 
        dataP(i,1)*dataP(i,2)   dataP(i,2)^2+L^2        dataP(i,2)*dataP(i,3)
        dataP(i,1)*dataP(i,3)   dataP(i,2)*dataP(i,3)   dataP(i,3)^2+L^2];
    
    Uppi = [0 -dataPp(i,3) dataPp(i,2)
            dataPp(i,3) 0 -dataPp(i,1)
            -dataPp(i,2) dataPp(i,1) 0];
    
    deltai = dataP(i,1:3)-dataPp(i,1:3);
    
    Ai = [M -M*Uppi ; -Uppi'*M Uppi'*M*Uppi];
    Atmp = Atmp + Ai;
    
    bi = [M*deltai' ; Uppi*M*deltai'];
    btmp = btmp + bi;
    
    % c_i = delta_i*M*delta_i';
    % c = c + c_i;
end

A = Atmp;
b = btmp;
