% MBICP3DGETAB -
% Compute the matrix A and b for the metric-based ICP 3D Biota_icra06
%
%% Description:
%
% Author - 	
%       Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%       Institut de Robotica i Informatica Industrial, CSIC-UPC
%       Last change: 07/2010
%       Version 1.2
%
% Usage
%
% Variables:
%   Inputs:
%     dataP  - a 3xn matrix, data 3D
%     dataPp - a 3xn matrix, data 3D
%     L      - a postive integer.
%   Outputs:
%
% Notes:
% Formulas are written as stated in Biotacra06, some similarities to the 
% variables are:
%
% dataP = z_t = S_ref = P_1
% dataPp = z_t+1 = S_new = P_2
%
% If you have trobles with your results try inverting the data when in you 
% define dataPp, it is:
%
% dataPp = z_t = S_ref = P_1 =  Xr
% dataP = z_t+1 = S_new = P_2 = Xq

%% Code
function [A b] = MbICP3DgetAb2(dataPp,dataP, L)


delta = dataP-dataPp;
L2 =L*L;

n1 = size(dataP,2);
M = zeros(3,3,n1);
M(1,1,:) = dataP(1,:).*dataP(1,:)+L2;
M(1,2,:) = dataP(1,:).*dataP(2,:);
M(1,3,:) = dataP(1,:).*dataP(3,:);
M(2,1,:) = M(1,2,:);
M(2,2,:) = dataP(2,:).*dataP(2,:)+L2;
M(2,3,:) = dataP(2,:).*dataP(3,:);
M(3,1,:) = M(1,3,:);
M(3,2,:) = M(2,3,:);
M(3,3,:) = dataP(3,:).*dataP(3,:)+L2;


Upp = zeros(3,3,n1);
%Uppi = [0 -dataz datay; dataz 0 -datax;-datay datax  0];
Upp(1,1,:) = zeros(1,n1);
Upp(1,2,:) = -dataPp(3,:);
Upp(1,3,:) = dataPp(2,:);
%
Upp(2,1,:) = dataPp(3,:);
Upp(2,2,:) = Upp(1,1,:) ;
Upp(2,3,:) = -dataPp(1,:);
%
Upp(3,1,:) = -dataPp(2,:);
Upp(3,2,:) = dataPp(1,:);
Upp(3,3,:) =Upp(1,1,:);

A11= sum(M,3);
A12 = sum(arrayMatrices3Dmultiplication(-M,Upp),3);
A21 = arrayMatrices3Dmultiplication(Upp,M);
A22 = sum(arrayMatrices3Dmultiplication(-A21,Upp),3);

A = zeros(6,6);
b = zeros(6,1);

A = [A11 A12; sum(A21,3) A22];
b = [sum(arrayMatricesVect3Dmultiplication(M,delta),2);
     sum(arrayMatricesVect3Dmultiplication(A21,delta),2)];