% Compute the derivatives from the metric based ICP Biota_icra06 to compute
% the covariance according with Censi

syms x y z tnx tny tnz;
syms L pix piy piz  ;
syms pixp piyp pizp  ;
syms Mi Upip Ai Bi Ci;


pi = [pix; piy; piz];
pip = [pixp; piyp; pizp];
pit = [pix piy piz];
pipt = [pixp piyp pizp];
deltai = (pi-pip);
deltait = (pit-pipt);
k = pix^2+ piy^2+ piz^2+L^2;

q = [x; y; z; tnx; tny; tnz];
qt = [x y z tnx tny tnz];


Mi = [pix^2+L^2 pix*piy  pix*piz
      pix*piy   piy^2+L^2 piy*piz
      pix*piz   piy*piz   piz^2+L^2];

Upip =[ 0    -pizp  piyp
        pizp     0 -pixp
       -piyp  pixp    0 ];

Upipt = [ 0      pizp  -piyp
         -pizp     0    pixp
          piyp  -pixp    0 ];
      
I = eye(3); 

Ai = simplify([Mi -Mi*Upip; -Upipt*Mi Upipt*Mi*Upip]);


Ait2 = [Ai(1,1) Ai(2,1) Ai(3,1) Ai(4,1) Ai(5,1) Ai(6,1)
    Ai(1,2) Ai(2,2) Ai(3,2) Ai(4,2) Ai(5,2) Ai(2,6)
    Ai(1,3) Ai(2,3) Ai(3,3) Ai(4,3) Ai(5,3) Ai(3,6)
    Ai(1,4) Ai(2,4) Ai(3,4) Ai(4,4) Ai(5,4) Ai(4,6)
    Ai(1,5) Ai(2,5) Ai(3,5) Ai(4,5) Ai(5,5) Ai(5,6)
    Ai(1,6) Ai(2,6) Ai(3,6) Ai(4,6) Ai(5,6) Ai(1,6)   
    ];

bi = simplify([Mi*deltai ;Upipt*Mi*deltai]);

bit = [bi(1,1) bi(2,1) bi(3,1) bi(4,1) bi(5,1) bi(6,1)];

ci = deltait*Mi*deltai;

%% Metric based distance function
 
J = simplify(qt*Ai*q)-simplify(2*bit*q)-simplify(ci);

%% First derivatives dJ/dq

dJ.dx = diff(J,x);
dJ.dy = diff(J,y);
dJ.dz = diff(J,z);
dJ.dtnx = diff(J,tnx);
dJ.dtny = diff(J,tny);
dJ.dtnz = diff(J,tnz);

%% Second derivatives d2J/dzdq

d2J.dpixdx = diff(dJ.dx,pix);
d2J.dpiydx = diff(dJ.dx,piy);
d2J.dpizdx = diff(dJ.dx,piz);
d2J.dpixpdx = diff(dJ.dx,pixp);
d2J.dpiypdx = diff(dJ.dx,piyp);
d2J.dpizpdx = diff(dJ.dx,pizp);


d2J.dpixdy = diff(dJ.dy,pix);
d2J.dpiydy = diff(dJ.dy,piy);
d2J.dpizdy = diff(dJ.dy,piz);
d2J.dpixpdy = diff(dJ.dy,pixp);
d2J.dpiypdy = diff(dJ.dy,piyp);
d2J.dpizpdy = diff(dJ.dy,pizp);
%%

d2J.dpixdz = diff(dJ.dz,pix);
d2J.dpiydz = diff(dJ.dz,piy);
d2J.dpizdz = diff(dJ.dz,piz);
d2J.dpixpdz = diff(dJ.dz,pixp);
d2J.dpiypdz = diff(dJ.dz,piyp);
d2J.dpizpdz = diff(dJ.dz,pizp);
%%

d2J.dpixdtnx = diff(dJ.dtnx,pix);
d2J.dpiydtnx = diff(dJ.dtnx,piy);
d2J.dpizdtnx = diff(dJ.dtnx,piz);
d2J.dpixpdtnx = diff(dJ.dtnx,pixp);
d2J.dpiypdtnx = diff(dJ.dtnx,piyp);
d2J.dpizpdtnx = diff(dJ.dtnx,pizp);
%%

d2J.dpixdtny = diff(dJ.dtny,pix);
d2J.dpiydtny = diff(dJ.dtny,piy);
d2J.dpizdtny = diff(dJ.dtny,piz);
d2J.dpixpdtny = diff(dJ.dtny,pixp);
d2J.dpiypdtny = diff(dJ.dtny,piyp);
d2J.dpizpdtny = diff(dJ.dtny,pizp);
%%

d2J.dpixdtnz = diff(dJ.dtnz,pix);
d2J.dpiydtnz = diff(dJ.dtnz,piy);
d2J.dpizdtnz = diff(dJ.dtnz,piz);
d2J.dpixpdtnz = diff(dJ.dtnz,pixp);
d2J.dpiypdtnz = diff(dJ.dtnz,piyp);
d2J.dpizpdtnz = diff(dJ.dtnz,pizp);

%% Visualize 
fprintf('\n d2J.dpixdx = '); disp(d2J.dpixdx)
fprintf('d2J.dpiydx = '); disp(d2J.dpiydx) 
fprintf('d2J.dpizdx = '); disp(d2J.dpizdx)
fprintf('d2J.dpixpdx = '); disp(d2J.dpixpdx)
fprintf('d2J.dpiypdx = '); disp(d2J.dpiypdx)
fprintf('d2J.dpizpdx = '); disp(d2J.dpizpdx) 
%%

fprintf('\n d2J.dpixdy = '); disp(d2J.dpixdy) 
fprintf('d2J.dpiydy = '); disp(d2J.dpiydy) 
fprintf('d2J.dpizdy = '); disp(d2J.dpizdy) 
fprintf('d2J.dpixpdy = '); disp(d2J.dpixpdy)
fprintf('d2J.dpiypdy = '); disp(d2J.dpiypdy)
fprintf('d2J.dpizpdy = '); disp(d2J.dpizpdy)


fprintf('\n d2J.dpixdz = '); disp(d2J.dpixdz)
fprintf('d2J.dpiydz = '); disp(d2J.dpiydz)
fprintf('d2J.dpizdz = '); disp(d2J.dpizdz)
fprintf('d2J.dpixpdz = '); disp(d2J.dpixpdz)
fprintf('d2J.dpiypdz = '); disp(d2J.dpiypdz)
fprintf('d2J.dpizpdz = '); disp(d2J.dpizpdz)


fprintf('\n d2J.dpixdtnx = '); disp(d2J.dpixdtnx)
fprintf('d2J.dpiydtnx = '); disp(d2J.dpiydtnx)
fprintf('d2J.dpizdtnx = '); disp(d2J.dpizdtnx)
fprintf('d2J.dpixpdtnx = '); disp(d2J.dpixpdtnx)
fprintf('d2J.dpiypdtnx = '); disp(d2J.dpiypdtnx)
fprintf('d2J.dpizpdtnx = '); disp(d2J.dpizpdtnx)

fprintf('\n d2J.dpixdtny = '); disp(d2J.dpixdtny)
fprintf('d2J.dpiydtny = '); disp(d2J.dpiydtny)
fprintf('d2J.dpizdtny = '); disp(d2J.dpizdtny)
fprintf('d2J.dpixpdtny = '); disp(d2J.dpixpdtny)
fprintf('d2J.dpiypdtny = '); disp(d2J.dpiypdtny)
fprintf('d2J.dpizpdtny = '); disp(d2J.dpizpdtny)



fprintf('\n d2J.dpixdtnz = '); disp(d2J.dpixdtnz)
fprintf('d2J.dpiydtnz = '); disp(d2J.dpiydtnz)
fprintf('d2J.dpizdtnz = '); disp(d2J.dpizdtnz)
fprintf('d2J.dpixpdtnz = '); disp(d2J.dpixpdtnz)
fprintf('d2J.dpiypdtnz = '); disp(d2J.dpiypdtnz)
fprintf('d2J.dpizpdtnz = '); disp(d2J.dpizpdtnz)