%% ICP3DMATCHING
%
%% Description
%
% Author - 	
%       Ernesto Homar Teniente Aviles
% 			CONACYT Scholarship
%       Institut de Robotica i Informatica Industrial, CSIC-UPC
%       Last change - 11/2010
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
%
% Usage:
%   [nnpts flag] = icp3DMatching(refPts,queryPts[,OPTIONS])
%
% Variables
%
% Inputs :
% 	refPts    -
%   queryPts  -
%
%  Options:
%   'Normals'
%   'PlanesError'
%   'NNSearch'
%   'DistanceFilterSigma'
%   'ErrorMetric'
%   'CurrentIteration'-  current iteration
%   'Iteration' - number of iterations , it is used with 'CurrentIteration' 
%    to apply the normals filter in the first iterarions
% Outputs:
%   nnpts   - a nx7 matrix - pairpoints (:,1:6), distances (:,7)
%
% 
% Options:
%

%% Code

function [nnpts flagC varargout]= icp3DMatching(refPts,queryPts,varargin)


[dimRef  szRef] = size(refPts);
[dimQuery szQuery] = size(queryPts);

if dimRef~=dimQuery
    error('Invalid dimension compatibility between data and query points')
end

if (dimQuery < 3 || dimRef < 3)
   error('Mismatching dimentions for the query or reference data')
end

chkvarargin(varargin,2);
flag = [0 0 0];

[ nnsearch,      ...
  matchmetric,   ...
  dsigma_,        ...
  dfixed,        ...
  refNormals,    ...
  queryNormals,  ...
  maxAngle,      ...
  refPlanesError, ...
  planesTreshold, ...
  mincorr,       ...
  cleanfilter,   ...
  c_iter,          ...
  boxsize,       ...
  dmetric,       ...
  L,             ...
  refData,       ...
  savematch,     ...
  iter] = process_options(varargin,                ...
                          'NNSearch', 'GLTree',       ...
                          'MatchMetric', 0,           ...
                          'DistanceFilterSigma', [],  ...
                          'DistanceFilterFixed', [],  ...
                          'RefNormals',[],            ...
                          'QueryNormals', [],         ...
                          'MaxAngle',deg2rad(45),     ...
                          'PlanesError', [],          ...
                          'MaxPlanesErr', 10000,      ...
                          'MinCorrespondences',250,   ...
                          'UseFilterBeforeIteration',100, ...
                          'CurrentIteration', 1000,   ...
                          'BoxSize',2,                ...
                          'DistanceMetric', 'Euclidean',    ...
                          'L', [],                    ...
                          'ReferenceBox',[],          ...
                          'SaveMatch',[],             ...
                          'Iteration', 0);

if ~isempty(refPlanesError)
  flag(1,3) =1;
end
if ~isempty(refNormals)
  flag(1,1) =1;
end
if ~isempty(queryNormals)
  flag(1,2) =1;
end

% Verify there is a path to save the data for matching later ECMR_11
do_savematch= false;
if ~isempty(savematch)
  if ~isempty(savematch{:,2})
    do_savematch = true;
  end
end

% Compute nearest neighbors
% Return the index (nninx) for the refPts correspondet with the
% i-esim point of queryPts, d is the distance between them.
if isempty(nnsearch)
  nnsearch = 'GLTree';
end

% Todo change all this for a simple function called getKnn, so code will be cleaner
switch nnsearch 
  case 'ANN'
    [nninx nnD]= annquery(refPts, queryPts, 1,'search_sch','fr','radius',1);
%     [nninx nnD]= annquery(refPts, queryPts, 1);
    % Recover the neigbors in refPts fro the NN found as the new refPts
    nninx = buildCorrIndex(nninx, nnD);
    [ignore inx ] =find(nninx(1,:)>0);
    nninx = nninx(:,inx);
  case 'GLTree'
    ptrtree=BuildGLTree3D(refPts);
    [nninx nnD]=KNNSearch3D(refPts,queryPts,ptrtree,1);% Return the correspondig index in ref for the query
    DeleteGLTree3D(ptrtree);
    nninx = nninx';
    % Recover the neigbors in refPts fro the NN found as the new refPts
    nninx = buildCorrIndex(nninx, nnD);
  case 'BNN'
    if ~isempty(refData)
      if do_savematch % old line isempty(savematch)
        nninx = boxesNNMatch(refPts,refData,queryPts,boxsize,dmetric,L);
      else
        [nninx rboxes] = boxesNNMatch(refPts,refData,queryPts,...
                                      boxsize,dmetric,L);
      end
    else
      error('Missing information,  ReferenceBox for boxesNNMatch')
    end
%     nninx = boxesNNMa(refPts,queryPts,boxsize,dmetric,L);  
  otherwise
    error('Invalid kNNmethod')
end

  if ~(sum(flag)) && matchmetric
    error('Missing all information for metric points to planes')
  end


  % Filter the repeated points returned by the NN only for the pt2pt metric

  if matchmetric == 0 || matchmetric ==1
    filRepPts =0;
    if filRepPts && iter<=cleanfilter % usin mindist filter

      if do_savematch% if the ir a path  
        [nninx, rboxes]= icpFilterRepeatedIndex(nninx,rboxes);
      else
        nninx = icpFilterRepeatedIndex(nninx);
      end
    end
  end

% Match script 
if ~isempty(nninx)
   % normals filter
  
  if ~isempty(maxAngle) && flag(1,1)&&flag(1,2) 
    %if flag(1,1)&&flag(1,2)
      % verify if the matrix containig the normals exist
      % else warning('Non existen normals, skiping normals filtering')
      if iter <= cleanfilter && iter~=0
        if flag(1,2)
          if szQuery > szRef   
            refNormals_ = getDataFromIndex(refNormals,nninx(1,:));
            if strcmp(nnsearch,'BNN')
              queryNormals_ = getDataFromIndex(queryNormals,nninx(2,:));
            else
      %         queryNormals_ = queryNormals;
              queryNormals_ = getDataFromIndex(queryNormals,nninx(2,:));
            end
          else
            refNormals_ = refNormals(:,nninx(1,:));
             if strcmp(nnsearch,'BNN')
              queryNormals_ = queryNormals(:,nninx(2,:));
             else
      %          queryNormals_ = queryNormals;
               queryNormals_ = queryNormals(:,nninx(2,:));
            end
          end
          inx = unitVectorFilter(refNormals_,queryNormals_,maxAngle);
          % kepp only those indexs from ANN that passed the normal test

          nninx =nninx(:,inx);  
          if do_savematch
           rboxes =  rboxes(inx,:);
          end
        end
      end
    else
          warning('Unable to run normals filter');
   % end
  end


  switch matchmetric
    case 0 % pt2pt
       % Filter the repeated points returned by the NN only for the pt2pt metric
       filRepPts =1;
       if filRepPts && iter<=cleanfilter % usin mindist filter
         if do_savematch % if the 
           [nninx, rboxes]= icpFilterRepeatedIndex(nninx,rboxes);
         else
           nninx = icpFilterRepeatedIndex(nninx);
         end
       end
       nnpts = icp3Dpt2ptMetric(refPts,    ...
                               queryPts , ...
                               nninx);
    case 1 % pt2plane
      % select the points that have passed the tests
     if flag(1,1)&&flag(1,3)
        if szQuery > szRef
          refPtsNN = getDataFromIndex(refPts,nninx(1,:));
          if ~isempty(refPlanesError)
            planErrorNN = getDataFromIndex(refPlanesError,nninx(1,:));
          end
          if flag(1,2)
            refNormals = getDataFromIndex(refNormals,nninx(1,:));
          end
            queryPtsNN  = getDataFromIndex(queryPts,nninx(2,:));
        else
          refPtsNN = refPts(:,nninx(1,:)) ;
          if ~isempty(refPlanesError)
            planErrorNN = refPlanesError(:,nninx(1,:));
          end
          if ~flag(1,2)
            refNormals = refNormals(:,nninx(1,:));
          end
          queryPtsNN = queryPts(:,nninx(2,:));
        end
  %       if szQuery > szRef   
  %           refNormals = getDataFromIndex(refNormals,nninx(1,:));
  %       else
  %           refNormals = refNormals(:,nninx(1,:));
  %       end
        % compute the proyected points
        if nargout == 3
          [nnpts, inx]= icp3Dpt2plMetric( refPtsNN,     ...
                                  queryPtsNN,   ...
                                  planErrorNN,  ...
                                  refNormals,   ...
                                  planesTreshold, ...
                                  nninx);
           if ~isempty(inx)
             nninx = nninx(:,inx);
             rboxes = rboxes(:,inx);
           end
        else
          nnpts = icp3Dpt2plMetric( refPtsNN,     ...
                                  queryPtsNN,   ...
                                  planErrorNN,  ...
                                  refNormals,   ...
                                  planesTreshold,...
                                  nninx);
        end
     else
       if ~flag(1,3)
         error('Missing information to perform pt2planes, no planesError defined')
       elseif ~flag(1,1)
         error('Missing information to perform pt2planes, no reference Normals defined')
       else
         error('Missing information to perform pt2planes')
       end
     end
    case 3
    % Only planes
%       inx = find(nninx(3,:)<=0.8);
      %nninx = nninx(:,inx);
      nnpts = icp3Dpt2plMetric( refPts(:,nninx(1,:)),     ...
                                queryPts(:,nninx(2,:)),   ...
                                refPlanesError(:,nninx(1,:)),  ...
                                refNormals(:,nninx(1,:)),   ...
                                planesTreshold,nninx, ...
                                'OnlyPlanes',1);
    otherwise
      error('Invalid matching metric')
  end


  % if dfixed
  %   [x inx] = find(nnpts(7,:)<=dfixed);
  %   nnpts = nnpts(:,inx);
  % %   nninx = ;
  % end
   if ~isempty(dfixed)
     inx = find(nnpts(7,:)<=dfixed);
     nnpts = nnpts(:,inx);
   end


  if dsigma_
    % modificar para regresar los indices para que acepte maxima y minima
    % sigma
    if ~do_savematch;% ~isempty(savematch)
      nnpts =filPtsMaxDistSigma(nnpts,dsigma_(1));
    else
      % for the BNN
      [nnpts inx] =filPtsMaxDistSigma(nnpts,dsigma_(1));
      rboxes = rboxes(inx,:);
      nninx = nninx(:,inx);
    end
  end

  flagC =1;
  if size(nnpts,2)<=mincorr
    warning('No enough correspondeces')
    flagC =0;
    % TODO: print a warning and make icp to end process returning the las
    % good pose
  end


  if do_savematch;% ~isempty(savematch)
    vars{1} = 'SaveDir';  % x(1)
    vars{2} = 'IDr';      % x(2)
    vars{3} = 'IDq';      % x(3)
    x = isstrincell(vars,savematch);
    % Verifying SaveMatch Parameters
    if x(1)
      if x(2)
        if x(3)
          s = filesep;
          dir_ = [savematch{x(1)*2} s 'pairs'];
          if ~exist(dir_,'dir')
              mkdir(dir_);
          end
          file_ = ['nnbox_match_'  ...
                    num2str(savematch{x(2)*2}, '%04.0f') '_'...
                    num2str(savematch{x(3)*2}, '%04.0f') '.txt'];
          fid = fopen(file_,'w');
          % nninx (rinx,qinx)
          for i=1:size(nninx,2)
              fprintf(fid, '%d\t%d\t%d\t%d\t%d\n',nninx(1:2,i),rboxes(i,:));
          end
          fclose(fid);
          movefile(file_ ,dir_);
        end
      end
    end
  end

  if nargout == 3
    varargout(1) = {nninx};
  end
else
  nnpts =[];
  flagC =0;
  if nargout == 3
    varargout(1) = {[]};
  end
  warning('No correspondeces');
end
  
