%% ICP3DMATCHING
%
%% Description
%
% Author - 	
%       Ernesto Homar Teniente Aviles
% 			CONACYT Scholarship
%       Institut de Robotica i Informatica Industrial, CSIC-UPC
%       Last change - 11/2010
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
%
% Usage:
%   [nnpts flag] = icp3DMatching(refPts,queryPts[,OPTIONS])
%
% Variables
%
% Inputs :
% 	refPts    -
%   queryPts  -
%
%  Options:
%   'Normals'
%   'PlanesError'
%   'NNSearch'
%   'DistanceFilterSigma'
%   'ErrorMetric'
%  
% Outputs:
%   nnpts   - a nx7 matrix - pairpoints (:,1:6), distances (:,7)
%
% 
% Options:
%

%% Code

function [nnpts flagC varargout]= icp3DMatching2(refPts,queryPts,varargin)


[dimRef  szRef] = size(refPts);
[dimQuery szQuery] = size(queryPts);

if dimRef~=dimQuery
    error('Invalid dimension compatibility between data and query points')
end

if (dimQuery < 3 || dimRef < 3)
   error('Mismatching dimentions for the query or reference data')
end

chkvarargin(varargin,2);
flag = [0 0 0];

[ nnsearch      ...
  matchmetric   ...
  dsigma        ...
  dfixed        ...
  refNormals    ...
  queryNormals  ...
  maxAngle      ...
  refPlanesError...
  planesTreshold...
  mincorr       ...
  cleanfilter   ...
  iter          ...
  boxsize       ...
  dmetric       ...
  L] = process_options(varargin,                ...
                          'NNSearch', 'GLTree',       ...
                          'MatchMetric', 0,           ...
                          'DistanceFilterSigma', [],  ...
                          'DistanceFilterFixed', [],  ...
                          'RefNormals',[],            ...
                          'QueryNormals', [],         ...
                          'MaxAngle',deg2rad(45),     ...
                          'PlanesError', [],          ...
                          'MaxPlanesErr', 10000,      ...
                          'MinCorrespondences',250,   ...
                          'UseFilterBeforeIteration',100, ...
                          'CurrentIteration', 1000,   ...
                          'BoxSize',2,                ...
                          'DistanceMetric', 'Euclidean',    ...
                          'L', []);

if ~isempty(refPlanesError)
  flag(1,3) =1;
end
if ~isempty(refNormals)
  flag(1,1) =1;
end
if ~isempty(queryNormals)
  flag(1,2) =1;
end


% Compute nearest neighbors
% Return the index (nninx) for the refPts correspondet with the
% i-esim point of queryPts, d is the distance between them.
if isempty(nnsearch)
  nnsearch = 'GLTree';
end

switch nnsearch 
  case 'ANN'
    [nninx nnD]= annquery(refPts, queryPts, 1);
    % Recover the neigbors in refPts fro the NN found as the new refPts
    nninx = buildCorrIndex(nninx, nnD);
  case 'GLTree'
    ptrtree=BuildGLTree3D(refPts);
    [nninx nnD]=KNNSearch3D(refPts,queryPts,ptrtree,1);
    DeleteGLTree3D(ptrtree);
    nninx = nninx';
    % Recover the neigbors in refPts fro the NN found as the new refPts
    nninx = buildCorrIndex(nninx, nnD);
  case 'BNN'
    nninx = boxesNNSearch(refPts,queryPts,boxsize,dmetric,L);  
  otherwise
    error('Invalid kNNmethod')
end

if flag(1,1)&&flag(1,2)&& iter <= cleanfilter
  % verify if the matrix containig the normals exist
  % else warning('Non existen normals, skiping normals filtering')
  if flag(1,2)
    if szQuery > szRef   
      refNormals_ = getDataFromIndex(refNormals,nninx(1,:));
      if strcmp(nnsearch,'BNN')
        queryNormals_ = getDataFromIndex(queryNormals,nninx(2,:));
      else
        queryNormals_ = queryNormals;
      end
    else
      refNormals_ = refNormals(:,nninx(1,:));
       if strcmp(nnsearch,'BNN')
        queryNormals_ = queryNormals(:,nninx(2,:));
       else
         queryNormals_ = queryNormals;
      end
    end
    inx = unitVectorFilter(refNormals_,queryNormals_,maxAngle);
    % kepp only those indexs from ANN that passed the normal test
    nninx =nninx(:,inx);      
  else
    warning('Unable to run normals filter');
  end
end


if ~(sum(flag)) && matchmetric
  error('Missing all information for metric points to planes')
end


% Filter the repeated points retuned by ANN only for the pt2pt metric
filRepPts =1;

if filRepPts && iter<=cleanfilter % usin mindist filter
%    inx = sortrows(inx')';
%    [u m n] = unique(inx(1,:));
%    inxF = filRepInx(inx(1,:)); 
%    inx = inx(:,m);
  inx = icpFilterRepeatedIndex(nninx);
  nninx = inx;
end
varargout(1) = {nninx};
switch matchmetric
  case 0 % pt2pt
    nnpts = icp3Dpt2ptMetric(refPts,    ...
                             queryPts , ...
                             nninx);
  case 1 % pt2plane
   if flag(1,1)&&flag(1,3)
      if szQuery > szRef
        refPtsNN = getDataFromIndex(refPts,nninx(1,:));
        if ~isempty(refPlanesError)
          planErrorNN = getDataFromIndex(refPlanesError,nninx(1,:));
        end
        if flag(1,2)
          refNormals = getDataFromIndex(refNormals,nninx(1,:));
        end
          queryPtsNN  = getDataFromIndex(queryPts,nninx(2,:));
      else
        refPtsNN = refPts(:,nninx(1,:)) ;
        if ~isempty(refPlanesError)
          planErrorNN = refPlanesError(:,nninx(1,:));
        end
        if ~flag(1,2)
          refNormals = refNormals(:,nninx(1,:));
        end
        queryPtsNN = queryPts(:,nninx(2,:));
      end
%       if szQuery > szRef   
%           refNormals = getDataFromIndex(refNormals,nninx(1,:));
%       else
%           refNormals = refNormals(:,nninx(1,:));
%       end

      nnpts = icp3Dpt2plMetric( refPtsNN,     ...
                                queryPtsNN,   ...
                                planErrorNN,  ...
                                refNormals,   ...
                                planesTreshold);
   else
     if ~flag(1,3)
       error('Missing information to perform pt2planes, no planesError defined')
     elseif ~flag(1,1)
       error('Missing information to perform pt2planes, no reference Normals defined')
     else
       error('Missing information to perform pt2planes')
     end
   end
  otherwise
    error('Invalid matching metric')
end




% if dfixed
%   [x inx] = find(nnpts(7,:)<=dfixed);
%   nnpts = nnpts(:,inx);
% %   nninx = ;
% end

if dsigma
  % TODO: modificar para regresar los indices
  nnpts =filPtsMaxDistSigma(nnpts,dsigma);
end
flagC =1;
if size(nnpts,2)<=mincorr
  warning('No enough correspondeces')
  flagC =0;
  % TODO: print a warning and make icp to end process returning the las
  % good pose
end


