%% filANN
% function to filter the index returned by ANN, searching an unique NN for
% each query/model and reference point

%% Description
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 09/2008
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
% Usage
%
% Variables:
%   Input:
%       indexa      - a n1 x 1 vector of integer, complement data.
%       indexb      - a n2 x 1 vector of integers, data to be filtered.
%       distance    - a real, distance between ponits.
%       switchIbIa  - a boolean, decides wich row to filter
%       maxDis      - a real, maximum valid distance
%
%   Outputs:
%       filnnid - a matrix formed by [index1 index2 distance], returns the 
%                 filtered index accordigly with the value of switchIbIa
%% Code
function filnnidx = filANN(indexa,indexb,distance, switchIbIa,maxDis,varargin)


if nargin == 4

    maxDis  = 10000;

end

    

if switchIbIa > 1
    error('Invalid Value for switchIbIa, chose only 0 or 1')
end

j = 1;
filnnidx = [];
m1 = size(indexa,1);
for i = 1:m1-1
   if indexb(i) == indexb(i+1) 
       if distance(i) < distance(i+1) 
           if switchIbIa == 0
               filnnidx(j,:) = [indexa(i+1) indexb(i+1) distance(i+1)];
           elseif switchIbIa == 1
               filnnidx(j,:) = [indexb(i+1) indexa(i+1) distance(i+1)];
           end
       end
   else
       if distance(i) < maxDis 
           if switchIbIa == 0
               filnnidx(j,:) = [indexa(i) indexb(i) distance(i)];
           elseif switchIbIa == 1
               filnnidx(j,:) = [indexb(i) indexa(i) distance(i)];
           end
           j = j+1;
       end
   end
end