%% saveICPtoDat
% Function to save the Smart data in pairs, ICP data and original data in
% .dat files
% Accordingly with the ETHZ .dat file specs. :
% a 0 before the data means Vehicle Position
% a 1 before the data meand Laser Impact
%
%% Description
%	saveICPtoDat(file, poses, data, ICPdata,inis, lasts, fdate)
%
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 02/09/2008
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
%
% Usage:
% 
%       saveICPtoDat(file, poses, ICPdata)
%
% Variables:
%
% 	Inputs:
%		file - a string, name for the .dat file
% 		poses - a matrix n x 6, robot poses
%		data  - struct(n).data with a m x 3 matrix. structures containing the data
%		inis  - a string, first cloud 
%		lasts - a string, last cloud 
%		fdate - a string, containg the date yymmddThhmmss
%
%	Notes:
%		the saved file will have the 
%
%% Code
function saveICPtoDat(file, poses, ICPdata, varargin)
% function saveICPtoDat(file, poses, data, ICPdata,inis, lasts, fdate,
% varargin)

poses_n = size(poses,1)/2;
poses_n = size(poses,1);
ICPdata_m = size(ICPdata,2);
pause(0.1)
if poses_n ~= ICPdata_m
    error('Pose number mismatch with data number')
end
% fdate = datestr(now,30);
% filename = [file '_' fdate inis lasts];
filename = file;
% if varargin > 3
fiddat = fopen([filename '.dat'],'w');
% fdate = datestr(now,30); % if necessary
i=1;
fprintf('Saving file : %s.dat\n',filename);
for j=1:ICPdata_m
    fprintf('Saving ICP cloud %d \n',j);
    % add the 0 before de pose to indicate that this are pose values
    if 3== size(poses,2)
        fprintf(fiddat,'0 %f %f %f \n', [poses(i,:)]);
    else
        fprintf(fiddat,'0 %f %f %f %f %f %f \n', [poses(i,:)]);
    end
        
    for k =1:size(ICPdata(j).data,1)
        fprintf(fiddat,'%5.3f\t%5.3f\t%5.3f\n', [ICPdata(j).data(k,:)]);
    end
%     i=i+2;
    i=i+1;
end
fclose(fiddat);
% movefile([filename '_ICP.dat'],'smart-upc.data.ICP');
fprintf('File SAVED \n')

%% Saving the original selcted data
% fiddat = fopen([filename '.dat'],'w');
% i=1;
% data_m = size(data,2);
% fprintf('Saving file : %s.dat\n',filename);
% for j=1:data_m
%     fprintf('Saving cloud %d \n',j);
%     % add the 0 before de pose to indicate that this are pose values
%     fprintf(fiddat,'0 %f %f %f %f %f %f\n', [poses(j,:)]);
%     for k =1:size(data(j).data,1)
%         fprintf(fiddat,'1 %f %f %f \n', [data(j).data(k,:)]);
%     end
%     i=i+2;
% end
% fclose(fiddat);
% movefile([filename '.dat'],'smart-upc.data.ICP');
% fprintf('File SAVED \n')