%% filMostRepData
% Filter the most x representative points of a correpondeces set
% accordingly with the distance helped with an histogram of the distance

%% Description
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 5/2008
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
%
% Usage
%       filtdata = filtMostRepData(data,dcol,porcentage,plot,fig)
%
% Variables:
%   Input: 
%       data        - a n x m matrix, with the data to filter
%       dcol        - a integer indicating in wich colum is the distance
%       between the correspondeces
%       percent     - a decimal, between 0-1 indicating the porcentage of
%       the final data
%       dhist       - an integer, need for the histogram see hist help for
%               more information (numero de bins del histograma)
%       plot        - a number, indicating when to plot the histograms
%
%   Outputs:
%       filtdata - a m x 3 matrix, most representative porcentage data
%% Code
function filtdata = filtMostRepData(data,dcol,percent,dhist,plot)

[histo(1,:) histo(2,:)] = hist(data(:,dcol),dhist);
%[histo(1,:) histo(2,:)] = hist(pairedIndex(:,3),dhist);
if plot
    figure(plot);
    hold on;
    subplot(2,2,3,'restore');
    title('Distances Histogram')
    bar(histo(2,:),histo(1,:));
 end
% Only the xx% of the neares neigbors is taken.
%%
filthist= filtHist(histo, size(data,1), percent);
%%
% using the filtered histogram find the most representative data
filtdata= porcentFil(data,filthist,histo,dcol);
%%
[histo(1,:) histo(2,:)] = hist(filtdata(:,dcol),dhist);
if plot
    subplot(2,2,4,'restore');
    title('Distances Histogram')
    bar(histo(2,:),histo(1,:));
end



