
% Options
%   'NumberofNN' - +integer, number of nearest neighbors, include the query
%                   point it self, this due to ANN lib
%                   Default - 13 
%                   Minimum - 5
%    'Iterations' - +integer, number of times to perform the filter
%                   Default - 5
%

function [points varargout] = filterbyNNMeanDistance (qPts, meanDistD,varargin)


chkvarargin(varargin,2);

% Default values
[NNN        ...
 max_iter   ...
 qnormals   ...
 qplerror   ...
 qdiscontinuity ...
 qcurvature ...
 qlamda_3   ...
 sigma      ...  
 nns      ] = process_options(varargin,     ...
                      'NumberofNN',13,      ...
                      'Iterations', 1,       ...
                      'Normals', [],        ...
                      'PlanesError',[],     ...
                      'Discontinuity', [],  ...
                      'Curvature',[],       ...
                      'Lamda_3',[],         ...
                      'Sigma', 1.5,         ...
                      'NNS', 'gltree');

if NNN < 3
  error('Invalid number of nearest neighbors')
end


n = size(qPts,2);

fprintf('Nearest neighbors distance filter :\n')
filter = 1;
iter =0;

while filter
  switch nns
    case 'ann'
      [nninx nnD]= annquery(qPts, qPts, NNN);
    case 'gltree'
      ptrtree=BuildGLTree3D(qPts);
      [nninx nnD]=KNNSearch3D(qPts,qPts,ptrtree,NNN);
      DeleteGLTree3D(ptrtree); 
      nnD = nnD';
  end
  dm=  getDistanceANNMean(nnD,nns);
  if max(1.1*dm) > meanDistD
    iter = iter+1;
    a = 0;
    fprintf('\titeration : %d\n', iter);    
    [dm sortInx] = sort(dm);
    % Sort the points acording with previus sorting
    qPts = qPts(:,sortInx);
    % filter adjust nn by mean with a maximum desired distance
    inx = appBinSearch(dm,0.001,meanDistD);
    qPts = qPts(:,1:inx);
    if ~isempty(qplerror)
      qplerror = qplerror(:,sortInx);
      qplerror = qplerror(:,1:inx);
    end
    if ~isempty(qnormals)
      qnormals = qnormals(:,sortInx);
      qnormals = qnormals(:,1:inx);
      a =a+1;
    end
    if ~isempty(qlamda_3)
      qlamda_3 = qlamda_3(:,sortInx);
      qlamda_3 = qlamda_3(:,1:inx);
      a = a+2;
    end
    if ~isempty(qdiscontinuity)
      qdiscontinuity = qdiscontinuity(:,sortInx);
      qdiscontinuity = qdiscontinuity(:,1:inx); 
      a = a+4;
    end
    if ~isempty(qcurvature)
      qcurvature = qcurvature(:,sortInx);
      qcurvature = qcurvature(:,1:inx); 
      a = a+8;
    end

    % find the new NN
    if iter >= max_iter
      filter = 0;
    end
  else
    filter = 0;
  end
end

switch nns
  case 'gltree'
   
end
fprintf('Cloud  Size :  Original %d, sampled : %d \n', [n size(qPts,2)]);

points = qPts;





switch a
  case 1
    varargout(1) = {qnormals};
  case 2
    varargout(1) = {qplerror};
  case 3
    varargout(1) = {qnormals};
    varargout(2) = {qplerror};
  case 4
    varargout(1) = {qdiscontinuity};
  case 7 
    varargout(1) = {qnormals};
    varargout(2) = {qplerror};
    varargout(3) = {qdiscontinuity};
    varargout(4) = {qlamda_3};
end




function distMean =  getDistanceANNMean(nnd,nns)

    [r c] = size(nnd);
    
%     tmp = mean(nnd(2:r,:));
%     distMean = mean(tmp);
  switch nns
    case 'ann'
      distMean  = mean(nnd(2:r,:));
    case 'gltree'
      distMean  = mean(nnd(1:r-1,:));
  end
