function L3D = ReadLaserScans3D(L,varargin)
%
% intputs:
%   L - string, laser scan 3d file name
%   Parameters:
%     1 - string, 'points'. Return only 3d points whit out normals
% 
% outputs:
%   L3D. - struct
%        .points  - a 3 x n matrix, laser scan 3D points
%        .normals - a 3 x n matrix, 3D points normals
%
%%


fidtxt = fopen(L ,'r');


 switch nargin
   case 1
     readNormals = 1;
   case 2
     switch varargin{1}
       case 'points'
         readNormals = 0;
       otherwise
         readNormals = 1;
     end
   otherwise
     error('Wrong number of input arguments')
 end
fprintf(1,'Reading laser scans 3D file %s....',[L '.txt']);
while ~feof(fidtxt)
    fidread = textscan(fidtxt,'%s {',1);
    dataType = fidread{1};
    %display(dataType);
    fidread = textscan(fidtxt,'%n','delimiter', '}');
    tmp = fidread{:};
    switch char(dataType);
      case 'points'
        n_points = (size(tmp,1)-1)/3;
        L3D.points = zeros(3,n_points);
        for i=1:n_points
          ii = i*3;
          L3D.points(:,i) = tmp(ii-2:ii);
        end
      case 'normals'
        if readNormals
          n_normals = (size(tmp,1)-1)/3;
          L3D.normals = zeros(3,n_normals);
          for i=1:n_normals
            ii = i*3;
            L3D.normals(:,i) = tmp(ii-2:ii);
          end
        end
      case 'errorAdjPlanes'
        %TODO.
    end
end

fclose(fidtxt);
if n_points~=n_normals
  L3D.points = [];
  L3D.normals = [];
  error('There is a problem in your file, number of points and normals are not equal');
end
fprintf(1,'DONE\n');
    

