% BOXESSAMPLEROLD
% Uniform Sampling using Boxes with an specified box size
%
%% Description
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Scholarship
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 11/2010
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
% Usage 
% 	SampledData = UnifSampBoxes(data,boxsize,bMedian)
%
% Variables:
%
% Inputs;
%   data    - a nx3 matrix, data to be sampled
%   boxsize - a integer, size of the box
%
% Outputs :
%   SampledData - a mx3 matrix, sampling result

%% Code
function SampledData = boxesSamplerOld(data,boxsize, bMedian)

if ~exist('bMedian')
    bMedian = 1;
end
fprintf('USBox...')
n = size(data,1);

% boxsize = 0.50; % 50 centimetros


% Find de minimun x, y and z to create the cube
minx=min(data(:,1))-0.01;
miny=min(data(:,2))-0.01;
minz=min(data(:,3))-0.01;
% Find de minimun x, y and z to create the cube
maxx=max(data(:,1))+0.01;
maxy=max(data(:,2))+0.01;
maxz=max(data(:,3))+0.01;

% Find how many boxes do we need per each axis
boxesx = ceil((maxx-minx)/boxsize);
boxesy = ceil((maxy-miny)/boxsize);
boxesz = ceil((maxz-minz)/boxsize);
boxes(boxesx,boxesy,boxesz).n=0;
count =1;

for i=1:n
    % is which box is that point?
    boxx = ceil((data(i,1)-minx)/boxsize);
    boxy = ceil((data(i,2)-miny)/boxsize);
    boxz = ceil((data(i,3)-minz)/boxsize);
    if(boxes(boxx,boxy,boxz).n)% boxes(boxx,boxy,boxz).n is there a point
%         boxes(boxx,boxy,boxz).p=(boxes(boxx,boxy,boxz).n*boxes(boxx,boxy,boxz).p+data(i,:)')/(boxes(boxx,boxy,boxz).n+1);
%         boxes(boxx,boxy,boxz).n=boxes(boxx,boxy,boxz).n+1;
    else
        boxes(boxx,boxy,boxz).p=data(i,:)'; % boxes(boxx,boxy,boxz).p stores the points in that box
        boxes(boxx,boxy,boxz).n=1;
        count = count+1;
    end
end


SampledData = zeros(3,count);

% Sampled Data stores only the median values of those boxes which have
% points
j=1;
for x=1:boxesx
  for y=1:boxesy
    for z = 1:boxesz
      if (boxes(x,y,z).n)
%         SampledData(j,:) = [ x y z];
        SampledData(:,j) = boxes(x,y,z).p;
        j=j+1;
%         SampledData = [SampledData; boxes(x,y,z).p'];
      end
    end
  end
end

fprintf('DONE \n')
