% ERASELOWER3DPOINTS
% Erase points that are down a threshold from the sensor origin 
%
%% Description
% Author - 
%       Ernesto Homar Teniente Aviles
%       CONACYT Scholar
%       Institut de Robotica i Informatica Industrial, CSIC-UPC
%       Last change: 11/2010
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
%
% Usage:
%       fil_data = tUpZ(data,pose, max_z)
%
% Variables:
%   Input:
%       data    - a 3xn matrix, data to sample
%       axis    - a char - working axis 'x', 'y' or 'z'
%       threshold  - a real
%
%     Options:
%       'SensorOrigin' - 3x1 vector, sensor origin coordinates
%
%   Outputs:
%       fil_data - a 3xn matrix, sampled data

%% Code
function [fil_data, varargout] = eraseLowerPoints3D(data,axis,threshold,varargin)

origin = process_options(varargin,       ...
                          'SensorOrigin', [0;0;0]);

switch axis
  case 'x'
    axis_ = 1;
    threshold = origin(1)-threshold;
  case 'y'
    axis_ = 2;
    threshold = origin(2)-threshold;
  case 'z'
    axis_ = 3;
    threshold = origin(3)-threshold;
end

inx = data(axis_,:)>=threshold;
fil_data = data(:,inx);
if nargout>=2
  [~,inx_] = find(inx==1);
  varargout(1) = {inx_};
end
% data = sortrows(data',axis_);
% min_d = (threshold + origin(axis_)) ;
% delta = 0.05;
% 
% [low_inx upp_inx]= binSearch(min_d,delta,0, data(:,axis_));
% n = size(data,1);
% fil_data = data(upp_inx:n,:)';

