% SAMPLEFILTERBOXES
% This function is used to sample and filter 3D data using a boxes list
% is an extention of sampleFilterBoxes with new features like filtering
% using the number of neighbor boxes, also returns corresponding box number
% for each point and the index to its neighbors
%% Description:
%
% Author - 	Ernesto H. Teniente A.
%           CONACYT Scholar
%           Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change:
%               Date: 11/2010
%               Author: Ernesto H. Teniente A.
%			          Version 1.0
% 
% This code has been developed as part of the XXXX Project
% XXX-YYY-000000
%
% Usage:
%		[newpts [OPTIONS]] = points3DBoxesOperations(pts,boxsz[,OPTIONS])
%
% Inputs:
%     pts   -3xn matrix , 3D data to sample
%     boxsz - +real, box size to sample and filter
%
%  Options:
%     'ComputeNormals' - a bool
%     'SampleType' - a string,  'Median' or 'Mean'
%     'FilterPointsintheBox' - +integer, minimum number of points in each
%                   box, if normal is asked defaul value is set to 3
%                   otherwise is set to 1
%     'FilterNeighborBoxes' -+integer, minimum number of boxes around
%                   box(x,y,x). TODO
%
% Outputs:
%     newpts - mx3 matrix
%  
%   Options:
%     normal -  mx3 matrix, computed normal for each box
%     planesError - 1xm vector, computed plane fit error for the normal
%     boxes(m) - a struct with field with size m
%              index   - current box index [x y z]
%              neindex -  neighbors index
%
%% CODE



function [newpts varargout] = points3DBoxesOperations(pts,boxsz,varargin)

fprintf('sampleFilterBoxeslist: ...')

chkvarargin(varargin,2);
tic
[sampletype ...
  min_boxpts...
  neigh_boxn,...
  computenormal,...
  retBox] = process_options(varargin,   ...
                  'SampleType', 'Median',   ...
                  'FilterPointsintheBox',1, ...
                  'FilterNeighborBoxes',[],...
                  'GetNormals',0,...
                  'ReturnBoxes',0);

[ordereddata occupiedboxes] = boxesListandPtsOrdered(pts,boxsz);
dim = size(pts,1);

% TODO- box neigbors sampling
% TODO- +if we are going to compute the normals the we must change 
%       min_boxpts =3, to warantie that the normal can be computed
%       + also we can set a maximun numer to compute the normal because
%         there could be boxes with to many points that computing the
%         normal would be higly constly.


if computenormal
   min_boxpts =5;
end

% Filter by points in the box
if min_boxpts>1
  fprintf('...')
  % 
  r.list = find(occupiedboxes.npts(occupiedboxes.inx.list) >=min_boxpts);
  % Boxes that passed the test
  boxes = occupiedboxes.boxes(occupiedboxes.inx.list(r.list) ,:);
  % recompute occupied boxes
  ordereddata.pts = ordereddata.pts(:,r.list);
  % recover the new occupied boxes 
  occupiedboxes =  boxesOccupied(boxes);
  
end
n_boxes = size(occupiedboxes.boxes,1);

% % Filter by the number of neighbor boxes
% if ~isempty(neigh_boxn)
%   if neigh_boxn
%     depth = 1;
%     min_n_ne = 6;
%   else
%     vars{1} = 'Depth'; %  Depth for the neighbor boxes search
%     vars{2} = 'MinNumNeigBox' ; % minimun number of neighbor boxes
%     x = isstrincell(vars,neigh_boxn);
%     if x(1)
%       depth = neigh_boxn{x(1)*2};
%     else
%       depth = 1;
%     end
%     if x(2)
%       min_n_ne = neigh_boxn{x(2)*2};
%     else
%       min_n_ne = 6;
%     end
%   end
%   
%   [ne_boxes n_ne]= findNeighborBoxes(occupiedboxes.boxes,depth);
%   n_ne_passed = find(n_ne >=min_n_ne);
%   % se debe de hacer algo similar a lo anterior de filtrado por el numero
%   % de puntos
% end

newpts =  zeros(dim,n_boxes);
% Sampling strategie
switch sampletype
  case 'Median'
    fprintf('.')
    for i=1:n_boxes
      thisinx = ceil((occupiedboxes.inx.first(i)+occupiedboxes.inx.last(i))/2);
      newpts(:,i) = ordereddata.pts(:,thisinx);
    end
  case 'Mean'
    fprintf('.')
    for i=1:n_boxes
      pts =  ordereddata.pts(:,                          ...
                            occupiedboxes.inx.first(i): ...
                            occupiedboxes.inx.last(i));
      newpts(:,i) = mean(pts,2);
    end
  otherwise
    error('Invalid parameter for SampleType')
end

if computenormal
   fprintf('...')
    maxPtsNormals =30;
    nptsrest = occupiedboxes.npts > maxPtsNormals;
    normal =  zeros(dim,n_boxes);
    planerror = zeros(1,n_boxes);
    if nargout >=4
      curvature = zeros(1,n_boxes);
      if nargout >=5
       lamda_3 = zeros(1,n_boxes);
      end
    end
    for i=1:n_boxes
      pts =  ordereddata.pts(:,                          ...
                           occupiedboxes.inx.first(i): ...
                           occupiedboxes.inx.last(i));
      if nptsrest(i)
        pts = randomSamp(pts,maxPtsNormals);
      end
      [normal(:,i) p planerror(i) ...
        curvature(i) lamda_3(i)]= findPtsNormalCetto(pts,[0;0;0.5]);
    end
%     normal = normalsReHorient(normal,newpts);
    
    varargout(1) = {normal};
    varargout(2) = {planerror};
    if nargout >=4
      varargout(3) = {curvature};
    end
    if nargout >=5
      varargout(4) = {lamda_3};
    end
end
fprintf('\n')
toc

fprintf('DONE\n')