% REDUCTIONFILTER3D
% Reduction filter for 3D data 

%% Description
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Scholarship
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 5/2008
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
%
% Usage
%       s_red = reductionfilter3D(s,r)
%
% Variables:
%	Input:
%		s - a n x 3 matrix, data to be reduced
%		r - a float, radious of reduction
%
%	Output:
%		s_red - a m x 3 matrix, m < n, reduced data
%
%% Code

function s_red = reductionfilter3D(s,r)
%fprintf('Applying Reduction filter 3D...');
j = 1;

%compute distance to the origin, the sort
n = size(s,2);
[x inx] = sort(dist3D(s, zeros(3,n)));
s = s(:,inx);


p_0 = s(:,1);
p_sum = p_0;
k = 1;
dos_r = 2*r;

for i=2:n
    p = s(:,i);
    if dist3D(p,p_0) < dos_r
        p_sum =p_sum + p;
        k = k + 1;
    else
        s_red(:,j) = p_sum/k;
        j =j+1;
        p_0 = p;
        p_sum = p_0;
        k = 1;
    end
end
s_red(:,j) = p_sum/n;

% s_red_size = size(s_red,1);

% fprintf('DONE\n');

