% SAMPLEFILTERBOXES
% This function is used to uniform sample and filter 3D data using a boxes list
% It also serves to compute the normals using the unsampled points cloud
%
%% Description:
%
% Author - 	Ernesto H. Teniente A.
%           CONACYT Scholar
%           Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change:
%               Date: 11/2010
%               Author: Ernesto H. Teniente A.
%			          Version 1.0
% 
% This code has been developed as part of the XXXX Project
% XXX-YYY-000000
%
% Usage:
%		[newpts [OPTIONS]] = sampleFilterBoxes(pts,boxsz[,OPTIONS])
%
% Inputs:
%     pts   -3xn matrix , 3D data to sample
%     boxsz - +real, box size to sample and filter
%
%  Options:
%     'GetNormals' - a bool
%     'SampleType' - a string,  'Median' or 'Mean'
%     'FilterPointsintheBox' - +integer, minimum number of points in each
%                   box, if normal is asked defaul value is set to 3
%                   otherwise is set to 1
%     'FilterNeighborBoxes' -+integer, minimum number of boxes around
%                   box(x,y,x). TODO
%
% Outputs:
%     newpts - mx3 matrix
%  
%   Options:
%     normal -  mx3 matrix, computed normal for each box
%     planesError - 1xm vector, computed plane fit error for the normal
%

%% CODE



function [newpts varargout] = sampleFilterBoxes(pts,boxsz,varargin)

fprintf('sampleFilterBoxeslist: ...')

chkvarargin(varargin,2);
tic
[sampletype, ...
  min_boxpts, ...
  min_boxpts_n, ...
  knn_normal_method,...
  computenormal, ...
  normals_factor_r, ...
  viewpt,    ...
  knn_r,     ...
  knn] = process_options(varargin,   ...
                  'SampleType', 'Median',   ...
                  'FilterPointsintheBox',1, ...
                  'MinPointsinthebox2ComputeNormals', 4, ...
                  'NeighborSearch','boxes',               ... % knn_r
                  'GetNormals',0,                         ...
                  'NormalsSearchRadiusFactor',2.1,        ...
                  'SensorCenter', [0;0;0.5] ,             ...
                  'SearchRadius',[],                      ...
                  'knn',15);

[ordereddata occupiedboxes] = boxesListandPtsOrdered(pts,boxsz);
dim = size(pts,1);


% TODO- box neigbors sampling
% TODO- +if we are going to compute the normals the we must change 
%       min_boxpts =3, to warantie that the normal can be computed
%       + also we can set a maximun numer to compute the normal because
%         there could be boxes with to many points that computing the
%         normal would be higly constly.

% se usaba como filtrado donde se eligian el minimo numero de puntos en la
% caja , esto con el fin de quitar ruido, se ha de reimplementar.
if computenormal
    min_boxpts =1;
end

% if min_boxpts>1
%   fprintf('...')
%   r.list = find(occupiedboxes.npts(occupiedboxes.inx.list) >=min_boxpts);
%   % Boxes that passed the test
%   boxes = occupiedboxes.boxes(occupiedboxes.inx.list(r.list) ,:);
%   % recompute occupied boxes
%   ordereddata.pts = ordereddata.pts(:,r.list);
%   occupiedboxes =  boxesOccupied(boxes);
%   
% end
n_boxes = size(occupiedboxes.boxes,1);


newpts =  zeros(dim,n_boxes);
% Sampling strategie
tic
switch sampletype
  % Todo improve performance here 
  case 'Median'
    fprintf('.')
    thisinx = ceil((occupiedboxes.inx.first+occupiedboxes.inx.last)/2);
    newpts = ordereddata.pts(:,thisinx);
  case 'Mean'
    fprintf('.')
%     if ~ strcmp('boxes',knn_normal_method)
      for i=1:n_boxes
        newpts(:,i) = sum(ordereddata.pts(:,               ...
                              occupiedboxes.inx.first(i):   ...
                              occupiedboxes.inx.last(i)),2) ...
                              /occupiedboxes.npts(i);
        % newpts(:,i) = mean(ordereddata.pts(:,                      ...
        %                     occupiedboxes.inx.first(i): ...
        %                     occupiedboxes.inx.last(i)),2);                    
      end
%     end
   
    
  otherwise
    error('Invalid parameter for SampleType')
end
toc

if computenormal
  % TODO, select the methos, consitency by number of points in the box,
  % only knn, knn and boxes, consitency on both
  tic
  desired_pts_in_box =3;
  desired_pts_in_rknn = 6;
  % poder cambiar el radio cuando se llama la funcion, por default ponerlo
  % vacio para que no lo tome en cuenta
  if isempty(knn_r)
    box_sx2 = boxsz*boxsz;
    factor_ = normals_factor_r;%2.5; % This variable indicates the search radius for the nearest
               % neighbor searh when computing the normals, as a factor of
               % the box size
    knn_r = factor_*(3*box_sx2)^0.5;
  end
  % min_boxpts =6; % min number per box
  maxPtsNormals =25; % max points per box
 
  % Todo, for those boxes were exis less than 6 points, find knn in the original
  % points cloud if points are far away we shall not use them (filtering)
  % with this restriction we can use only knnr to compute the normalsid
  switch knn_normal_method
    case 'knnr'
       knn_r_filter = ones(1,n_boxes);
       normalmethod = ones(1,n_boxes);
       [nninx,nnd_]= knnGet(ordereddata.pts,newpts,'knn',knn);
       
    case 'hybrid'
      knn_r_filter = ones(1,n_boxes);
      normalmethod = ones(1,n_boxes);% flag as radius search

      if min_boxpts_n>desired_pts_in_box
        inx = occupiedboxes.npts >=min_boxpts_n;
        normalmethod(inx) = 0; % flag as boxes search
        % inx = find(normalmethod>0);
        inx = normalmethod>0;
        [nninx nnd_]= knnGet(ordereddata.pts,newpts(:,inx),'knn',knn);
      end
    case 'boxes'
      normalmethod = zeros(1,n_boxes);
      knn_r_filter = ones(1,n_boxes);
  end
   
    
  %% Todo: apply distance filter

  fprintf('...')

  nptsrest = occupiedboxes.npts > maxPtsNormals;
  normal =  zeros(dim,n_boxes);
  planerror = zeros(1,n_boxes);
  if nargout >=6
    lamda_3 = zeros(1,n_boxes);
    lamda_2 = zeros(1,n_boxes);
    lamda_1 = zeros(1,n_boxes);
    if nargout >=8
      nn_number = zeros(1,n_boxes);
      if nargout>=9
        f_mean_diff = zeros(1,n_boxes);
      end
    end
  end
  j = 1;
  for i=1:n_boxes
    compute_normal_i = 1;
    if normalmethod(i) % use knn method
       r_inx = find(nnd_(:,j)<=knn_r);
       % TODO:use de mean filter with the knn
       nn_number(i) = length(r_inx);
       if length(r_inx) >=desired_pts_in_rknn;
        pts = ordereddata.pts(:,nninx(r_inx,j));
       else
        knn_r_filter(i) =0; 
        compute_normal_i = 0;
       end
      j= j+1;
    else % use de points in the box
      if occupiedboxes.npts(i)< desired_pts_in_box
        compute_normal_i = 0;
        knn_r_filter(i) = 0;
      else
        pts =  ordereddata.pts(:,                          ...
                           occupiedboxes.inx.first(i): ...
                           occupiedboxes.inx.last(i));
        nn_number(i) = nptsrest(i);
        if nptsrest(i)
          pts = randomSamp(pts,maxPtsNormals);
          nn_number(i) = maxPtsNormals;
        end
      end

    end
    if compute_normal_i
      [normal(:,i), p,planerror(i), ...
        lamda_1(i),lamda_2(i),lamda_3(i)]= findPtsNormalCetto(pts,viewpt);
      % if strcmp('Mean',sampletype)
      %   newpts(:,i) = p;
      % end
      if nargout>=9
        f_mean_diff(:,i) = abs(dist3D(newpts(:,i),p));
      end
      pts =[];
    end
  end
%     normal = normalsReHorient(normal,newpts);
  toc
  inx = knn_r_filter==1;
  newpts = newpts(:,inx);
  varargout(1) = {normal(:,inx)}; % estimated normals
  if nargout >2
    varargout(2) = {planerror(:,inx)};
    if nargout >=6
      varargout(3) = {lamda_1(:,inx)}; % largest eigenvalues from the PCA
      varargout(4) = {lamda_2(:,inx)}; %
      varargout(5) = {lamda_3(:,inx)}; % smallest igenvalues from the PCA
      if nargout >=7
        varargout(6) = {occupiedboxes.boxes(inx,:)'}; % Return the number of the box
        if nargout >=8
          varargout(7) = {nn_number(:,inx)};% return number of neighbors
          if nargout>=9
            varargout(8) = {f_mean_diff(:,inx)};
          end
        end
      end
    end
  end
  
% Output is in next order:
% [points, normals, planerror,{lamda_1,lamda_2,lamda_3},occupiedboxes, ...
% nn_number]
else
  varargout(1:nargout) = cell(1,nargout) ;
  warning('No retuned vaues for varargaout')
end
fprintf('\n')


fprintf('DONE\n')