%% findPointNormal
% Finds the normal of a plane given a quantity of points,
% also return its mean point and the related error
%
%% Description
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 12/2008
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
%
% Modified by:
%           Sergi Foix Salmeron
%           PRE-JAE
%           Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change - July, 2009
%
% This code has been developed as part of the XXX Project
% 
%% Usage:
%   [normal varargout] = findPointNormal(points, varargin)
%
% Variables :
%
% Inputs:
%       points       - [ 3 x n ], points that define a plane
%       varargin:
%           type     - 'cetto' or 'hoppe'
% 
% Outputs:
%       normal           - [ 1 x 3 ], vector containing the normal
%       varargout:
%           medianPoint  - [ 1 x 3 ], vector containing the median from points
%           relatedError - scalar, accumulated plane distance error
% 

%% Code
function [normal varargout] = findPointNormal(points, varargin)

numPoints = size(points,2);
if numPoints < 3
   error('Not enough points to define a plane');
end

type = 'cetto';

narg = length(varargin);
for i=1:2:narg
    switch char(varargin(i))
        case 'type'
            type = varargin{i+1};
    end
end


numPoints = size(points,2);
if numPoints < 3
   error('Not enough points to define a plane');
end

medianPoint = mean(points,2); % median point of the plane
if strcmp(type,'cetto')
    % Andrade Method
    Q = points*points';
    q = sum(points,2);
    R = Q - q*q'/numPoints;
elseif strcmp(type, 'hoppe')
    %Hoppe Method
    for ii = 1:numPoints
        centeredPoints(:,ii) = points(:,ii) - medianPoint;
    end
    R = centeredPoints*centeredPoints';
else
    error('Incorrect type field, please check it out');
end

[U S V]= svd(R);
normal = V(1:3,end); % normal del plano

% Calculates the accumulated error of the median point and normal vector 
% related with the Set of Points that support the plane
relatedError = 0;
for ii=1:numPoints
	relatedError = relatedError + abs(normal'*(points(:,ii)-medianPoint));
end
relatedError = relatedError/numPoints;

% Output variables
if nargout == 2 
    varargout(1) = {medianPoint};
elseif nargout ==3
    varargout(1) = {medianPoint};
    varargout(2) = {relatedError};
end
%% Plots point and normal vector

% h1=figure; hold on; set(h1, 'WindowStyle', 'docked');
% plot3(points(1,:),points(2,:),points(3,:),'r.', 'DisplayName', 'Q'); % Plots the points that give support to the plane
% quiver3(p(1,1),p(2,1),p(3,1),n(1,1),n(2,1),n(3,1)); %plot normals
% hold off;
