%% FINDPOINTSNORMAL
% Given a 3D points set find the normal vector for each point acording its  
% k nearest neighbors
%
%% Description
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 12/2008
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
%
% Usage:
%   nnidx =findPointsNormal(queryPts[,OPTIONS])
%
% Variables
%  Inputs:
%       queryPts 	- a 3 x n matrix, points searching its neighbors
%
%   Options;
%     'NumberofNN' - +integer, number of nearest neighbors, include the 
%                   query point it self, this due to ANN lib
%                   Default - 13 
%                   Minimum - 3
%
% Outputs:
%       Data.points     - a 3xm matrix, Resulting points from filtering 
%       Data.normal     - a 3xm  matrix, Normal for each point
%       Data.adjError   - mx1 matrix an integer, adjust error to the fitted plane
%

%% Code
function  [points normals varargout ]= findPointsNormal(queryPts,varargin)



chkvarargin(varargin,2);

% Default values
 [NNN ...
  rd  ...
  nnsearch] = process_options(varargin,  ...
                      'NumberofNN',20,    ...
                      'SearchRadius',0.75, ...
                      'NNSearch', 'GLTree');

if NNN < 3
  error('Invalid number of nearest neighbors, minimum required is 3')
end


% fprintf('In findPointsNormals: ')
% anninx = annquery(queryPts, queryPts, NNN);


switch nnsearch 
  case 'ANN'
    [nninx nnD]= annquery(queryPts, queryPts,NNN,'search_sch','fr','radius',rd);
  case 'GLTree'
    ptrtree=BuildGLTree3D(queryPts);
    [nninx nnD]=KNNSearch3D(queryPts,queryPts,ptrtree,NNN);
    DeleteGLTree3D(ptrtree);
    nninx = nninx';
    nnD = nnD';
end
% So far program is only tested with GLTree because the compilation issues
% of ann in 64 bits
fprintf('Computing normals...')
[r1 c1] = size(queryPts);
% inx_ = struct('nnwrd',[]);
% inx_a = [];


% queryPts = queryPts(:,inx_a);

[r1 c1] = size(queryPts);
normals = zeros(r1,c1);

if nargout >=3
  planesError = zeros(1,c1);
  if nargout == 6
    lamda_1 = zeros(1,c1);% lamda 1
    lamda_2 = zeros(1,c1);% lamda 2
    lamda_3 = zeros(1,c1);% lamda 3
  elseif nargout >=4
    discontinuity = zeros(1,c1);
    if nargout ==5
     lamda_3 = zeros(1,c1);
    end
  end
end
    
for i =1:c1
%   c_inx = inx_(i).nnwrd';
  [normals(:,i),p , planesError(i) ...
    lamda_1(i),lamda_2(i), lamda_3(i)]= findPtsNormalCetto(queryPts(:, nninx(:,i)),[0;0;0.8]);
end


% % Find at least 5 points that are in the search radius
j =1;
for i= 1:c1
  inx = find(nnD(:,i)<=rd);
  if length(inx)>=5
   inx_a(j) = i;
   j = j+1;
  end
end

points = queryPts(:,inx_a);
normals = normals(:,inx_a);



if nargout >=3
  varargout(1) = {planesError(:,inx_a)};
  if nargout == 6
    varargout(2) = {lamda_1(:,inx_a)};% lamda 1
    varargout(3) = {lamda_3(:,inx_a)};% lamda 2
    varargout(4) = {lamda_3(:,inx_a)};% lamda 3
  elseif nargout >=4
    ratio_l3 =  lamda_3./(lamda_1+lamda_2+lamda_3);
    varargout(2) = {ratio_l3(:,inx_a)};
    if nargout ==5
     varargout(3) = {lamda_3(:,inx_a)};
    end
  end
end

fprintf('DONE\n')



