
% FINDPTSNORMALCETTO
% Finds the normal of a plane given np points based on the chapter written
% by Juan Andrade-cetto on the Objet recogniton book
%
%% Description
% Author - 	
%       Ernesto Homar Teniente Aviles
% 			CONACYT Scholarship
%			  Institut de Robotica i Informatica Industrial, CSIC-UPC
%       Last change: 09/2011
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
%
% Modified by:
%           Sergi Foix Salmeron
%           PRE-JAE
%           Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change - September, 2009
%
%% Usage:
%   [ normal Options ] = findPtsNormalCetto( points )
%
% Variables :
%
% Inputs:
%       points       - [ 3 x n ], points that define a plane
% 
% Outputs
%       normal       - [ 1 x 3 ], vector containing the normal
%	
% Options:
%       medianPoint  - [ 1 x 3 ], vector containing the median from points
%       relatedError - scalar, accumulated plane distance error

%% Code
function [normal varargout] = findPtsNormalCetto(points,varargin)

numPoints = size(points,2);
if numPoints < 3
   error('Not enough points to define a plane') 
end

if nargin == 2
  view_point = varargin{1};
  correct_normals = 1;
else
  correct_normals =0;
end

% median point of the plane
% medianPoint = mean(points,2);

Q = points*points';
q = sum(points,2);
R = Q - q*q'/numPoints;

medianPoint = q/numPoints;

% 
% [V1 D1] = eigs(R);
% normal = V1(:,3);

[V D] = eig(R);
normal = V(:,1);

% verify if the norm is equal to one
if norm(normal)>=1.000000000000001
   fprintf('CorrectNormal: %2.6f  %2.6f  %2.6f\n  norm:%2.8f\n ',normal,norm(normal))
end
% a =q==0;
% if sum(a)
%   if a(3)
%    [value inx] = min(D(1:2,1:2));
%    normal = V(:,inx(1));
%   end
% else
% normal = V(:,3);

if correct_normals
  local_point = view_point-medianPoint;
% n_ = norm(local_point);
% dot(normal,(local_point)/n_)
n_ = (local_point(1)^2+local_point(2)^2+local_point(3)^2)^0.5;
 if sum(normal.*((local_point)/n_),1) <= 0
    normal = -normal;
%     plot3DNormals(medianPoint,normal,'r');
 end
  

end


% Another way to find the normal
% D= eig(R);
% [value inx] = min(D);
% [V D]= eig(R);
% n1 = V(:,inx)  

% Computes the mean of the signed distance from the points to the computed plane
relatedError = 0;
for ii=1:numPoints
	relatedError = relatedError + abs(normal'*(points(:,ii)-medianPoint));
end
relatedError = relatedError/numPoints;

% relatedError2 = sum(abs(dot(normal(:,ones(numPoints,1)),...
%                                   points-medianPoint(:,ones(numPoints,1)))))...
%                                   /numPoints;


% RELATED ERROR USING PCA


% Output variables
if nargout >= 2 
    varargout(1) = {medianPoint};
  if nargout >=3
    varargout(2) = {relatedError};
    if nargout >=6
      varargout(3) = {D(3,3)};% lamda 1 for eigval
      varargout(4) = {D(2,2)};% lamda 2
      varargout(5) = {D(1,1)};% lamda 3 for eigval (smallest)
      if nargout ==8
        varargout(6) = {V(:,3)}; % eigvect lamda 1
        varargout(7) = {V(:,2)}; % eigvect lamda 2
      end
    end
  end
end

%% Plots point and normal vector

% h1=figure; hold on; set(h1, 'WindowStyle', 'docked');
% plot3(points(1,:),points(2,:),points(3,:),'r.', 'DisplayName', 'Q'); % Plots the points that give support to the plane
% quiver3(p(1,1),p(2,1),p(3,1),n(1,1),n(2,1),n(3,1)); %plot normals
% hold off;
