% FINDPTSNORMALHOPPE
% Finds the normal of a plane given numPoints points based on Hoppe_cg92 
% article
%
%% Description
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT scholarship
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 09/2011
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
%
% Modified by:
%           Sergi Foix Salmeron
%           PRE-JAE
%           Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change - September, 2009
%
%% Usage:
%   [ normal OPTIONS ] = findPtsNormalHoppe( points )
%
% Variables :
%
% Inputs:
%       points       - [ 3 x n ], points that define a plane
% 
% Outputs
%       normal       - [ 1 x 3 ], vector containing the normal
%	
% Options:
%       medianPoint  - [ 1 x 3 ], vector containing the median from points
%       relatedError - scalar, accumulated plane distance error

%% Code
function [normal varargout] = findPtsNormalHoppe(points,varargin)

numPoints = size(points,2);
if numPoints < 3
   error('Not enough points to define a plane') 
end

if nargin == 2
  view_point = varargin{1};
  correct_normals = 1;
else
  correct_normals =0;
end


%% Compute normal according with Hoppe_cg92

% median point of the plane
medianPoint = mean(points,2);

R = zeros(3);
for i = 1:numPoints
     points_centered = points(:,i)-medianPoint;
     R = R + points_centered*points_centered';
end
 
% [V D] = eigs(R);
% normal = V(:,3);
[V D] = eig(R);
normal = V(:,1);
% a =q==0;
% if sum(a)
%   if a(3)
%    [value inx] = min(D(1:2,1:2));
%    normal = V(:,inx(1));
%   end
% else
% normal = V(:,3);

if correct_normals
%   if medianPoint (2)>=0
%       view_point(2) = view_point(2)-1;
%   else
%       view_point(2) = view_point(2)+1;
%   end
  local_point = view_point-medianPoint;
%   plot3DNormals(medianPoint,normal,'b');
  if (dot(normal,(local_point)/norm(local_point))) <= 0
    normal = -normal;
%     plot3DNormals(medianPoint,normal,'r');
  end
end


% Another way to find the normal
% D= eig(R);
% [value inx] = min(D);
% [V D]= eig(R);
% n1 = V(:,inx)  

% Computes the mean of the signed distance from the points to the computed plane
relatedError = 0;
for ii=1:numPoints
	relatedError = relatedError + abs(normal'*(points(:,ii)-medianPoint));
end
relatedError = relatedError/numPoints;

% relatedError = sum(abs(dot(normal(:,ones(numPoints,1)),...
%                                   points-medianPoint(:,ones(numPoints,1)))))...
%                                   /numPoints;


% RELATED ERROR USING PCA


% Output variables
if nargout == 2 
    varargout(1) = {medianPoint};
elseif nargout ==3
    varargout(1) = {medianPoint};
    varargout(2) = {relatedError};
elseif nargout ==6
    varargout(1) = {medianPoint};
    varargout(2) = {relatedError};
%     varargout(3) = {D(1,1)};% lamda 1 for eigs
    varargout(3) = {D(3,3)};% lamda 1 for eig
    varargout(4) = {D(2,2)};% lamda 2
%     varargout(5) = {D(3,3)};% lamda 3 for eigs
    varargout(5) = {D(1,1)};% lamda 3 for eig
end