%% H2xyzrxryrz -
% Convert an homogeneus position matrix to a 6D position vector (x,y,z,rx,ry,rz)
%
%% Description
%
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change - Feb, 2009
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
%
% Usage:
%   poseeuler = H2xyzrxryrz(H)
%
% Variables
%   Inputs :
%       H   - 4 x 4 matrix, homogeneus transformation matrix
% 
%   Ouputs :
%       poseeuler - a 1 x 6 matrix, returns (x,y,z,roll,pitch,yaw) values
%                   roll: angle about the x axis, rx
%                   pitch: angle about the y axis, ry
%                   yaw: angle about the z axis, rz
%
% NOTE: keep in mind the angles description above

%% Code:
function poseeuler = H2xyzrxryrz(H)
poseeuler(1:3,1) = H(1:3,4)';
euler = R2rxryrz(H(1:3,1:3));
poseeuler(4,1) = euler(1,1);
poseeuler(5,1) = euler(2,1);
poseeuler(6,1) = euler(3,1);

