%% xyzRxRyRz2HRt
% Converts an Euler vector transformation in a Post-Multiplied Homogeneous
% Rotrotranslation transformation
%
%% Description
% 
% Author - Sergi Foix Salmerón
%          PRE-JAE 
%          Institut de Robotica i Informatica Industrial, CSIC-UPC
%          Last change - July, 2009
%
% This code has been developed as part of the XXX Project
% 
%% Usage
% 
% HRt1 = xyzRxRyRz2PosHRt( 0, 0, 0, 35, 60, -10, 'deg', 'pre');
% HRt2 = xyzRxRyRz2PosHRt( 3, 3, 3, pi/5, -pi/10, 0, 'rad', 'pre');
% pose3 = [ 5 5 5 90 30 45 ]';
% HRt3 = xyzRxRyRz2PosHRt( pose3, 'deg', 'pre');
% pose4 = [ 8 8 8 0 pi/3 pi/2 ]';
% HRt4 = xyzRxRyRz2PosHRt( pose4, 'rad', 'pre');
% plotPose(HRt1);plotPose(HRt2);plotPose(HRt3);plotPose(HRt4);

%% Variables:
% 
%   Input:
%       - varargin:
% 
%   Outputs:
%       - Rt:   Homogeneous Rototraslation Transformation
% 

%% Code

function [ HRt ] = xyzRxRyRz2HRt( varargin )

if (nargin==3) && (size(varargin{1},1)==6)
   Rt.x = varargin{1}(1);
   Rt.y = varargin{1}(2);
   Rt.z = varargin{1}(3);
   Rt.phi = varargin{1}(4);
   Rt.theta = varargin{1}(5);
   Rt.psi = varargin{1}(6);
   dataType = varargin{2};
   tranType = varargin{3};
elseif (nargin==8) && (size(varargin,1)==1)
   Rt.x = varargin{1};
   Rt.y = varargin{2};
   Rt.z = varargin{3};
   Rt.phi = varargin{4};
   Rt.theta = varargin{5};
   Rt.psi = varargin{6};
   dataType =varargin{7};
   tranType = varargin{8};
else
    error('Error:Variables','Error: Input variables are not correct\n Usage: xyzRxRyRz2HRt(x,y,z,Rx,Ry,Rz,''dataType'', ''tranType'')\n where dataType is either ''deg'' or ''rad''\n and tranType is either ''pre'' or ''pos''');
end

Tt=[1 0 0 Rt.x;
    0 1 0 Rt.y;
    0 0 1 Rt.z;
    0 0 0  1];

if strcmp(dataType,'deg')
    Tx=[ 1 0 0 0; 0 cosd(Rt.phi) -sind(Rt.phi) 0; 0 sind(Rt.phi) cosd(Rt.phi) 0; 0 0 0 1 ];
    Ty=[ cosd(Rt.theta) 0 sind(Rt.theta) 0; 0 1 0 0; -sind(Rt.theta) 0 cosd(Rt.theta) 0; 0 0 0 1 ];
    Tz=[ cosd(Rt.psi) -sind(Rt.psi) 0 0; sind(Rt.psi) cosd(Rt.psi) 0 0; 0 0 1 0; 0 0 0 1 ];
elseif strcmp(dataType,'rad')
    Tx=[    1           0           0           0;
            0      cos(Rt.phi) -sin(Rt.phi)     0;
            0      sin(Rt.phi)  cos(Rt.phi)     0;
            0           0           0           1];
    Ty=[ cos(Rt.theta)  0      sin(Rt.theta)    0;
            0           1           0           0;
        -sin(Rt.theta)  0      cos(Rt.theta)    0;
            0           0           0           1];
    Tz=[ cos(Rt.psi) -sin(Rt.psi)   0           0;
         sin(Rt.psi)  cos(Rt.psi)   0           0;
            0           0           1           0;
            0           0           0           1];
else
    error('Error:Variables','Error: You must define if Euler angles are in degres(deg) or radians(rad)');
end
if strcmp(tranType, 'pre')
    HRt=Tt*Tz*Ty*Tx; % RPY -> RxRyRz (PRE-MULTIPLIED)
elseif strcmp(tranType, 'pos')
    HRt=Tt*Tx*Ty*Tz; % RPY -> RxRyRz (POS-MULTIPLIED)
else
    error('Error:Variables','Error: You must define if Euler angles are in degres(deg) or radians(rad)');
end