% RTPOSE -
% Given an homogeneus tranformation matrix, transform a poses set.
%
%% Description
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 09/2008
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
%
% Usage:
%       posesH = RTpose(H,poses)
%
% Variables:
%   Input:
%       H       - a 4 x 4 matrix, transformation matrix
%       poses   - a n x 6 matrix, poses set (x,y,z,r,p,y), 
%                   According with Lavalle book, Planning Algorithms:
%					* r - roll, rotation angle about the x axis
%					* p - pitch, rotation angle about the y axis
%					* y - yaw, rotation angle about the z axis
%
%   Outputs:
%       posesH  - a n x 6 matrix, transformed poses set 
%                                (x y z roll pitch yaw)

%% Code
function posesH = RtHpose6D(H,poses)

n = size(poses,1);
posesH = zeros(n,6);
for i = 1:n
    % trasformation matrix from the pose i
    Hi = [rxryrz2R(poses(i,4),poses(i,5),poses(i,5)) poses(i,1:3)'; 0 0 0 1];
    HHi = H*Hi;
    rxryrz = R2rxryrz(HHi(1:3,1:3));
    posesH(i,:) = [HHi(1:3,4)' rxryrz(1,1) rzryrzy(2,1) rxryrz(3,1)];
end

