% ROTATIONTRANSLATIONPOSES -
% Given an homogeneus tranformation matrix, transform a poses set.
%
%% Description
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			    CONACYT Schoolar
%			      Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 07/2010
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
%
% Usage:
%       posesH = rotationTranslatioPoses(H,poses)
%
% Variables:
%   Input:
%       H       - a 4 x 4 matrix, transformation matrix
%       poses   - a n x 6 matrix, poses set (x,y,z,rx,ry,rx), 
%                   According with Lavalle book, Planning Algorithms:
%					* r - roll, rotation angle about the x axis
%					* p - pitch, rotation angle about the y axis
%					* y - yaw, rotation angle about the z axis
%
%   Outputs:
%       posesH  - a n x 6 matrix, transformed poses set 
%                                (x y z roll pitch yaw)

%% Code
function posesH = rotationTranslatioPoses(H,poses)

n = size(poses,1);
posesH = zeros(n,6);
for i = 1:n
    % trasformation matrix from the pose i
    Hi = xyzRxRyRz2H(poses(i,:))
    HHi = H*Hi;
    posesH(i,:) = H2xyzrxryrz(HHi);
end

