% R2RXRYRZ -
% Compute Euler angles rz(yaw), ry(pitch) and rx(roll) given a rotation 
% matrix
%
%% Description
%
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Scholarship
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change - 08, 2009
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
% 
% Usage:
%   euler_angles = R2rxryrz(R)
%
% Variables:
%   Input:
%       R - a 3x3 matrix, rotation matrix
%   Outputs:
%       rxryrz - a 3x1 vector,rx(roll), ry(pitch) and rz(yaw) computed 
%               angles
%
% Note:
%   The given rotation matrix must became from th form the mathematic
%   formula given by R = RzRyRx
%

%% Code
function rxryrz = R2rxryrz(R)

rz = atan2(R(2,1),R(1,1));
ry = atan2(-R(3,1),R(1,1)*cos(rz)+R(2,1)*sin(rz));
rx = atan2(R(3,2),R(3,3));

rxryrz = [rx;ry;rz];
