% EXPMAP2AXISANGLE
%
% Convert a exponential map so(3) (rotational vector)to axis angle, return an 
% axis angle vector 1x4 = [Angle eulerAxis]. Axis angle representation is 
% equivalent to the more concisce rotation vector representation.
%
%% Description
%
% Author - 	
%       Ernesto Homar Teniente Aviles
%       CONACYT Scholarship
%       Institut de Robotica i Informatica Industrial, CSIC-UPC
%       Last change: 08/2009
% 
% Usage
%		axis_angle =  rexpMap2AxisAngle(expo)
%
% Variables:
%   Input:
%       expo- a 3x1 vector, exponential map [theta*w_x;theta*w_y; theta*w_z]
%
%   Outputs:
%       axis_angle	- a 4x1 vector, [theta; w_x; w_y; w_z]
%
% NOTES: http://en.wikipedia.org/wiki/Rotation_representation_(mathematics)

%% Code
function axis_angle =  expMap2AxisAngle(expo)
%theta = norm(expo); % Se obtiene theta
theta = sqrt(expo(1)^2+expo(2)^2+expo(3)^2);
n_x = expo(1)/theta; % n_x
n_y = expo(2)/theta; % n_y
n_z = expo(3)/theta; % n_z
axis_angle =[theta; n_x; n_y; n_z];

% n_ are normalized so they make and unitary vector