% EXPMAP2Q
% Convert an so(3) exponential map to a quaternion
%
%% Description
%
% Author -
%         Ernesto Homar Teniente Aviles
%         CONACYT Fellow
%         Institut de Robotica i Informatica Industrial, CSIC-UPC
%         Last change: 09/2010
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
% 
% Usage
%   	q = expMap2q(rotVec)
%
% Varibles:
%   Input
%       rotVec 	- a 3x1 vector, rotation vector  or axis angle vector
%
%
%   Output:
%       q		- a 4x1 vector, quaternion
%

%% Code


function q = expMap2q(epsilon)

axis_angle =  expMap2AxisAngle(epsilon)

theta = axis_angle(1); % Se obtiene theta

w_x = axis_angle(2); % w_x
w_y = axis_angle(3); % w_y
w_z = axis_angle(4); % w_z

q0 = (theta/2);
q1 = sin(theta/2)*w_x;
q2 = sin(theta/2)*w_y;
q3 = sin(theta/2)*w_z;

q =[q0; q1; q2; q3]
