function plane_pts = createPtsPlane(a,b,c,d,varargin)
% CREATEPTSPLANE -
% Given the plane equation parameters a,b,c and d create a
% points plane
%
% Description
%
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 08/2009
% 
% Usage
%       [plane_pts] = createPtsPlane(a,b,c,d[,OPTIONS])
%
% Varibles:
%   Inputs
%       a,b,c,d - integers, plane equation parameters
% 
%   Outputs:
%       plane_pts - a 3 x n matrix, points in the same plane
%
%   Options:
%       'Distribution' - uniform or random
%
%
%   Note:
%       Plane parametric equation, ax + by + cz + d = 0

%% CODE

int_low = -10;
int_up = 10;
step = 0.1;
x=int_low:step:int_up;
[X,Y] = meshgrid(x);

Z=(d-a*X-b*Y)/c;

Distribution = 'uniform';
% Distribution = 'random';

i = 1;
switch Distribution
    case 'uniform'
        for j=1:10
           for k = 1:10
              p(:,i) =  [X(j,k),Y(j,k),Z(j,k)];
              i = i+1;
           end
        end
    case 'random'
        n_pts = 20;
        p = zeros(3,n_pts);
        n = size(x,2);
        j_indx = ceil(1 + (n-1) * rand(n_pts,1));
        k_indx = ceil(1 + (n-1) * rand(n_pts,1));
%         j_indx = sort(ceil(rand(n_pts,1)*n));
%         k_indx = sort(ceil(rand(n_pts,1)*n));
        for j=1:n_pts
            p(:,i) =  [X(j_indx(j),k_indx(j)),Y(j_indx(j),k_indx(j)),Z(j_indx(j),k_indx(j))];
            i = i+1;

        end
end

plane_pts = p;

