function plane_par = getPlaneEqParameters(P1,P2,P3)
%GETPLANEPARAMETERS - Given three points compute the paremeters a,b,c and d
% form the parametric equation of the plane
% 
%% Description
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 08/2009
% 
% Usage
%       plane_par = getPlaneParameters(pts)
%
% Varibles:
%   Inputs
%       P1 - 1 x 3 vector, first points
%       P2 - 1 x 3 vector, first points
%       P3 - 1 x 3 vector, first points
% 
%   Outputs:
%       plane_par - a 4 x 1 vector, plane parameters [a;b;c;d]
%
%   Options
%
%
%   Note:
%       Plane parametric equation, ax + by + cz + d = 0

%% CODE



normal = cross(P1-P2,P1-P3);

a = normal(1);
b = normal(2);
c = normal(3);
d = -(a*P1(1)+b*P1(2)+c*P1(3));

plane_par = [a;b;c;d];







end
