% QMINM
% Compute from a correlation matrix the corresponding quaternion
% representing a rotation betwen two data sets
%
%% Description:
%
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Scholarship
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 08/2009
%			Version 1.0
%
% Usage:
%       q_min = qMinm(S)
%
% Variables:
%   Inputs:
%       S	- 3 x 3 matrix, correlation matrix between two data sets
%
% 	Outputs:
%		q_min -  a 4x1 vector, unit quaternion from the minimization
%		process
%
%   
% Notes: According with paper_xx
% N = [ (S.xx+S.yy+S.zz)       (S.yz-S.zy)       (S.zx-S.xz)       (S.xy-S.yx)
%          (S.yz-S.zy)      (S.xx-S.yy-S.zz)     (S.xy+S.yx)       (S.zx+S.xz)
%          (S.zx-S.xz)         (S.xy+S.yx)    (-S.xx+S.yy-S.zz)    (S.yz+S.zy)
%          (S.xy-S.yx)         (S.zx+S.xz)       (S.yz+S.zy)
%          (-S.xx-S.yy+S.zz) ];

%% Code:

function q_min = qMinm(S)
% 
SS.xyx.a = S(1,2)+S(2,1);
SS.xyx.r = S(1,2)-S(2,1);
SS.yzy.a = S(2,3)+S(3,2);
SS.yzy.r = S(2,3)-S(3,2);
SS.zxz.a = S(3,1)+S(1,3);
SS.zxz.r = S(3,1)-S(1,3);



N = [S(1,1)+S(2,2)+S(3,3), 	SS.yzy.r, 	SS.zxz.r,	SS.xyx.r
  		SS.yzy.r,	S(1,1)-S(2,2)-S(3,3), 	SS.xyx.a, 	SS.zxz.a
  		SS.zxz.r,          SS.xyx.a,	-S(1,1)+S(2,2)-S(3,3), SS.yzy.a
  		SS.xyx.r,          SS.zxz.a,		SS.yzy.a,	-S(1,1)-S(2,2)+S(3,3)];
    
[V D]= eig(N);
% [value index] = max(D);

q_min = V(:,4);