function [ newPoint ] = applyHRt2point( point, HRt )
%APPLYHRT2POINT Applies a Rigid Transformation to a unique point
% 
%   1st -> Rotation
%   2nd -> Traslation
% 
%   Usage:
% 
%   function [ newPoint ] = applyHRt2point( point, HRt )
% 
%   Inputs:
%       - point -> [ 1 x 3 ], point
%       - HRt   -> [ 4 x 4 ], Homogeneous Rigid Transformation
% 
%   Output:
%       - newPoint -> [ 1 x 3 ], Transformed point
% 
%   Author:
%           Sergi Foix Salmeron
%           PRE-JAE
%           Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change - July, 2009
% 

newPoint = HRt(1:3,1:3) * point + HRt(1:3,4);

