% ROTATIONTRANSLATION3D
% Transform a set of data given an homogeneous matrix or a vector pose
%
%% Description
%
% Author -Ernesto Homar Teniente Aviles
%         CONACYT Fellow
%         Institut de Robotica i Informatica Industrial, CSIC-UPC
%         Last change: 07/2010
%
% Usage
%   outData = data3D2HRtdata3D(data,H)
%
% Variables:
% Inputs 
% 	H 		- a 4 x 4 matrix, homegneus transformation matrix
%           or 1 x 6 vector, pose vector (x,y,z,rx,ry,rz)
% 	data 	- a 3 x n or n x 3  matrix, data to transform
%
% Output
%   outData - a 3 x n matrix, containg the transformed data

%% Code
function outData = rotationTranslation3D(data,H)

[n m] = size(data);
[n1 m1] = size(H);

if m1 == 6
  pose = H;
  H = ([rxryrz2R(pose(1,4),pose(1,5),pose(1,6)) (pose(1,1:3)'); 0 0 0 1]);
end

if m==3
  data = data';
end

[n m] = size(data);
outData = zeros(3,m);
R =H(1:3,1:3);
t =H(1:3,4);
for i=1:m
    outData(:,i) = R*data(:,i) + t;
end
