% TRANSDATAGL3LOC - 
% Given a reference frame, transform two data sets to this reference frame
% and return the transformed data and the relative robot motion in local
% coordinates
%
%% Description
% Author - 	
%           Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change: 08/2009
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
%
% Usage:
%       dat1L pos1L dat2L pos2L RTdataNN1] =
%           transDatgl2loc(dat1, pos1, dat2, pos2)
% Variables:
%   Input:
%		dat1 - a n x 3 matrix, data set 1
%		pos1 - a 1 x 6 vector, pose 1
%		dat2 - a n x 3 matrix, data set 2
%		pos2 - a 1 x 6 vector, pose 2
%   Outputs:
%       dat1L - a n x 3 matrix, data set 1 on local coordinates
%       pos1L - a 1 x 6 vector, pose 1 on local coordinates
%       dat2L - a n x 3 matrix, data set 2 on local coordinates
%       pos2L - a 1 x 6 vector, pose 2 on local coordinates
%       RTdataNN
%
%% Code
function [dat1L pos1L dat2L pos2L RTdataNN1] = ...
    transDatgl2loc(dat1, pos1, dat2, pos2)


 % Step 1:
 % Compute from dataN pose the corresponding trasformation 
 % matrix 
poseS = size(pos1,2);
if poseS == 3
    RTdataN = ([rxryrz2R(0,0,0) (pos1(1,1:3)'); 0 0 0 1]);
elseif poseS == 6
  % RTdataN = ([rxryrz2R(pos1(1,6),pos1(1,5),pos1(1,4)) (pos1(1,1:3)'); 0 0
  % 0 1]);
    RTdataN = ([rxryrz2R(pos1(1,4),pos1(1,5),pos1(1,6)) (pos1(1,1:3)'); 0 0 0 1]);
end
 invRTdataN = inv(RTdataN);
 % Step: 2
 % Convert each cloud to local coordinates 
 % (robot coordinates)
 % First Cloud n = i(reference) 
 dat1L = RTHData(dat1,invRTdataN);
 % vPoseL - vehicle position on local coordinates
 pos1L(1,:) =  RtH2xyzrxryrz(invRTdataN*RTdataN); 
 % Step 3:
 % Second Cloud n1 = i+1(model)
 dat2L = RTHData(dat2,invRTdataN);
 % Step 4:
 % Find the transformation matrix that represents the motion
 % from dataN = dat1 to dataN+1 = dat2
 % Step 4.1: get the transformation matrix for dataN
 %       - got it in step 2               
 % Step 4.2: get the transformation matrix for dataN+1
 if poseS == 3
     RTdataN1 = [rxryrz2R(0,0,0) (pos2(1,1:3)'); 0 0 0 1];
 elseif poseS == 6
     RTdataN1 = [rxryrz2R(pos2(1,4),pos2(1,5),pos2(1,6)) (pos2(1,1:3)'); 0 0 0 1];
%     RTdataN1 = [rxryrz2R(pos2(1,6),pos2(1,5),pos2(1,4)) (pos2(1,1:3)'); 0 0 0 1];
 end
 % Step 4.3: get the transformation from dataN to dataN+1
 RTdataNN1 = invRTdataN*RTdataN1;
 pos2L(1,:) =  RtH2xyzrxryrz(RTdataNN1);  
