
#include <math.h>
#include "mex.h"
#include <stdio.h>
#include <iostream>  // I/O 
#include <fstream>   // file I/O
#include <vector>

using namespace std;


/**Variables declaration */
struct valsStruct{
  unsigned int inx;
  double value;
} min_t,max_t;

struct valsVector{
  vector<unsigned int> inx;
  vector<double> value;
};

struct points3D{
  vector<double> x;
  vector<double> y;
  vector<double> z;
};

 valsVector ans;
/**Function declaration */

mxArray *process(const mxArray *mxMA, const mxArray *mxMB, const mxArray *mxMC);
mxArray *returnValues(const mxArray *mxMA);

double  MbDist_i(vector<double> * qP_i,vector<double> * rP_i, int L2); /** Compute metricbased distance por a point*/
double  MbDist_i(double qx_i,double qy_i,double qz_i,
                  double rx_i,double ry_i,double rz_i,int L2); 

valsVector MbMinDist(points3D  qPts,points3D rPts,int L);/**find the correspoinding point with the minimum distance from each query point respect to reference points */

valsStruct min(double data[],unsigned int MAXELS ); /** Find the minimum value returning the index and value*/
valsStruct min(vector<double> data);

points3D RetrievePoints(double *ma, int sizeMa);

/** This function computes the minimum distance for a set of points 3D  using Metric base distance from {biota_06} */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) { 
    if (nrhs < 3)
       mexErrMsgTxt("Tree inputs requiered"); 
    else if(nrhs > 3)
      mexErrMsgTxt("Too many input arguments.");
    /** Returns a matrix 2xn:
        row 1 = indexs;
        row 2 = values;
    */
    plhs[0] = process(prhs[0],prhs[1],prhs[2]);
    plhs[1] = returnValues(prhs[0]);
}

/*************** functions *********************************************/

// main function
mxArray *process(const mxArray *mxMA, const mxArray *mxMB, const mxArray *mxMC)
{

  
  double *ma = (double *)mxGetPr(mxMA);
  double *mb = (double *)mxGetPr(mxMB);
  double *mc = (double *)mxGetPr(mxMC);
  
  const int *maSize = mxGetDimensions(mxMA);
  const int *mbSize = mxGetDimensions(mxMB);

  int out[2];
  out[0] = 1;
  out[1] = maSize[1];
  // output 
  mxArray *mxSal = mxCreateNumericArray(2, out, mxDOUBLE_CLASS, mxREAL);
  double *sal = (double *)mxGetPr(mxSal);
  
  /****************************************************************************/
  int L =*mc;
  double x; 
//   valsVector ans;
  
  points3D qPts = RetrievePoints(ma,maSize[1]);
  points3D rPts = RetrievePoints(mb,mbSize[1]);
  
  /*Compute the index and values for qith the minimum distance to each query point */
  ans =MbMinDist(qPts,rPts,L);

  for (int i = 0; i < out[1]; ++i){
    *(sal + i)= ans.inx[i];
    }
   return mxSal;
  /****************************************************************************/
}

mxArray *returnValues(const mxArray *mxMA){
    double *ma = (double *)mxGetPr(mxMA);
    const int *maSize = mxGetDimensions(mxMA);
    int out[2];
    
    out[0] = 1;
    out[1] = maSize[1];
    // output 
    mxArray *mxSal = mxCreateNumericArray(2, out, mxDOUBLE_CLASS, mxREAL);
    double *sal = (double *)mxGetPr(mxSal);
    for(unsigned int i=0;i<out[1];i++){
      *(sal+i)= ans.value[i];
    }
//     *(sal)= 1;
    return mxSal;
};

double  MbDist_i(double qx_i,double qy_i,double qz_i,
                  double rx_i,double ry_i,double rz_i,int L2){

  //Variables declaration
  double d_i;
  double d_[2],q_i[2],r_i[2];
  double M[3][3];
  double k,k_,L_i;
  
  //delta
  d_[0] = rx_i-qx_i;
  d_[1] = ry_i-qy_i;
  d_[2] = rz_i-qz_i;
  // k 
  k_ =  sqrt(qx_i*qx_i+ qy_i*qy_i+ qz_i*qz_i);
  k = k_*k_+L2;
  L_i = L2/k;
  // Compute M
    
  M[0][0] = rx_i*rx_i/k+L_i;
  M[0][1] = rx_i*ry_i/k;
  M[0][2] = rx_i*r_i[2]/k;
  M[1][0] = M[0][1];
  M[1][1] = ry_i*ry_i/k+L_i;
  M[1][2] = ry_i*rz_i/k;
  M[2][0] = M[0][2];
  M[2][1] = M[1][2];
  M[2][2] = rz_i*rz_i/k+L_i;
  
  // Computing metric base distance
  d_i = d_[0]*(M[0][0]*d_[0]+M[0][1]*d_[1]+M[0][2]*d_[2])+
      d_[1]*(M[1][0]*d_[0]+M[1][1]*d_[1]+M[1][2]*d_[2])+
      d_[2]*(M[2][0]*d_[0]+M[2][1]*d_[1]+M[2][2]*d_[2]);
 return  d_i;
}

valsVector MbMinDist(points3D qPts,points3D rPts,int L) {
  vector<double> d_i;
  valsVector min_v;
  valsStruct min_l;
  unsigned int sizeQ= qPts.x.size();
  unsigned int sizeR= rPts.x.size();
  min_v.inx.resize(sizeQ);
  min_v.value.resize(sizeQ);
  int L2 = L*L;
  
  d_i.resize(rPts.x.size());

  for(unsigned int i=0;i<sizeQ;i++){
    //mexPrintf("query %d...",i);
    for(unsigned int j=0;j<sizeR;j++){
      d_i[j] = MbDist_i(qPts.x[i],qPts.y[i],qPts.z[i],
                        rPts.x[j],rPts.y[j],rPts.z[j],L2);
//   mexPrintf("inx (%d,%d) \n",i,j);   
    }
    // mexPrintf("DONE\n");
    min_l = min(d_i);
    min_v.inx[i] = min_l.inx+1;
    min_v.value[i] = min_l.value;
//     mexPrintf("inx: %d, val: %f\n",min_v.inx[i] ,min_v.value[i]);
    }
//   mexPrintf("Done min\n");
  return min_v;

};

valsStruct min(double vals[],unsigned int MAXELS){
   min_t.value = vals[0];
   min_t.inx = 0;
  for (unsigned int i = 1; i < MAXELS; i++){
    if (vals[i] <= min_t.value){
      min_t.inx =i;
      min_t.value = vals[i];
    }
  }
  return min_t;
}

valsStruct min(vector<double> vals){
   min_t.value = vals[0];
   min_t.inx = 0;
  for (unsigned int i = 1; i < vals.size(); i++)
    if (vals[i] <= min_t.value){
      min_t.inx =i;
      min_t.value = vals[i];
    }
  return min_t;
}

points3D RetrievePoints(double *ma, int sizeMa){
  
  points3D pts;
  pts.x.resize(sizeMa);
  pts.y.resize(sizeMa);
  pts.z.resize(sizeMa);
  for (unsigned int i=0;i<sizeMa;i++){
    pts.x[i] = *(ma + 0 + i*3);
    pts.y[i] = *(ma + 1 + i*3);
    pts.z[i] = *(ma + 2 + i*3);
    //mexPrintf("Point i: =%d\t%f\t%f\t%f\n",i,pts.x[i],pts.y[i],pts.z[i]);
  }
  
  return pts;
}