function C = boxesNNSearch(rPts,qPts,boxsize,varargin)
% Inputs:
% 
%   rPts  - 3xn matrix, box sorted reference points
%   qPts  - 3xn matrix, unsorted query points
%   rInfo - struct, box structure information 
%   rInx  - a 1xn vector, corresponding indixing in to the unsorted 
%           reference points
%   boxsize -  +real
%
%   Options:
%     3.- forcebounds - a struct,obligatories bound limits
%     1.- metric      - a string, point to point metric
%     2.- L           - a +real, Metric base mandatory pameter, if you use
%                       it with other Mteric base function it must be 
%                       consistent
%
%  Output:
%     C - a 3xm matrix, indices refered to the unsorted reference and query
%         points. 
%           C(1,:) - reference points indices
%           C(2,:) - query points indices
%           C(3,:) - distance between points indices

metric = 'Euclidean';
L = 3;
if nargin > 3
  metric = varargin{1};
  if ~strcmp(metric, 'Mb')
    metric = 'Euclidean';
  end
end
if nargin > 4;
  L = varargin{2};
end

% Retrieve boxStruture for rInfo
rData.bounds = boxBounds([rPts qPts]);
[p tmp rInx] = boxesListandPtsOrdered(rPts,boxsize,rData.bounds);
rData.B = tmp.npts;
rData.Ic = tmp.inx.first;
rData.I = tmp.boxes;
rPts = p.pts;
rData.nboxes = boxesQuantity(rData.bounds.delta,boxsize);
%% todo filter boxes with to few points
rBox.I = zeros(rData.nboxes(1)+1,rData.nboxes(2)+1,rData.nboxes(3)+1);
rBox.B = zeros(rData.nboxes(1)+1,rData.nboxes(2)+1,rData.nboxes(3)+1);
% Fill boxes with corresponding data
for i=1:size(rData.I,1)
    rBox.I(rData.I(i,1),rData.I(i,2),rData.I(i,3)) = rData.Ic(i);
    rBox.B(rData.I(i,1),rData.I(i,2),rData.I(i,3)) = rData.B(i);
end
% find qData boxes
[p tmp qInx] = boxesListandPtsOrdered(qPts,boxsize,rData.bounds);
qPts = p.pts;
qData.I =tmp.boxes;
qData.B = tmp.npts;
qData.Ic = tmp.inx.first;


% find NN
C =[];
mqPts= 5;ceil(7*log(boxsize*10.5620));
% if mqPts>2
%   mqPts = mqPts-2;
% end
mrPts= ceil(mqPts/3);
for i=1:size(qData.I,1);
  inx_= rBox.I(qData.I(i,1),qData.I(i,2),qData.I(i,3));
  if inx_
    if  mrPts <rBox.B(qData.I(i,1),qData.I(i,2),qData.I(i,3)) 
      if mqPts<tmp.npts(i)
%       fprintf('rBox.B  %d, qNPts:%d\n', rBox.B(qData.I(i,1),qData.I(i,2),qData.I(i,3)), ...
%       tmp.npts(i));
      rPts_inx = inx_:(inx_+rBox.B(qData.I(i,1),qData.I(i,2),qData.I(i,3))-1);
      rPts_ = rPts(:,rPts_inx);
      qPts_inx = qData.Ic(i):qData.Ic(i)+qData.B(i)-1;
      qPts_ = qPts(:,qPts_inx);
      xC = nnSearchBasic_c(rPts_,qPts_,metric,L);
      % Recover original index
      C = [C [ rPts_inx(xC(1,:));qPts_inx;xC(3,:)]];
      
      % Visual validation
%         O = rData.bounds.min'+ ...
%              [(qData.I(i,1)-1)*boxsize, ...
%               (qData.I(i,2)-1)*boxsize, ...
%               (qData.I(i,3)-1)*boxsize];
%         voxel(O,[boxsize boxsize boxsize],'r',0.01);
%         plotData3D(rPts_,'MarkerSize',7);
%         plotData3D(qPts_,'Color','r','Marker','*');
%         ptsC = [rPts_(:,xC(1,:));qPts_(:,xC(2,:))];
%         plotCorrespondences3D(ptsC);
%       end
      end
    end
  end
end
% plotCorrespondences3D([rPts(:,C(1,:)); qPts(:,C(2,:))])
% Se pretenden regresar los indices de las nuves originales no los de las
% ordenadas
C(1,:) = rInx(C(1,:));
C(2,:) = qInx(C(2,:));


