function h=PlotPoseSequence(S,color,arrows)
% Plots a sequence of poses.
%
% Plots a cell array of poses representing a trajectory (either ground
% truth trajectories or noisy trajectories resulting from the integration
% of sensor readings).
%   
% Parameters
%     S: The cell array with the sequence of poses.
%     color: The color for the line representing the trajectory.
%     arrows: True if an arrow has to be added to each pose representing
%             the orientation.
%
% Returns the handler of the line representing the sequence of poses.

  n=size(S,2);
  
	X=zeros(1,n);
	Y=zeros(1,n);
  
  for i=1:n
    X(i)=get(S{i},'x');
    Y(i)=get(S{i},'y');
  end
  
  hold on
	h=line(X,Y);
	set(h, 'marker','o' ,'markersize',3); %3
	set(h, 'color',color);

	if (arrows)
    d=0;
    for i=2:n
      X(i)=get(S{i},'x');
      Y(i)=get(S{i},'y');
      d=d+norm(X(i)-X(i-1),Y(i)-Y(i-1));
    end
    d=d/(n-1); % average displacement (used to plot orientation arrows of
               % a reasonable size)
             
		for i=1:n
			o=get(S{i},'orientation');
			ax=d*cos(o);
			ay=d*sin(o);
			h=quiver(X(i),Y(i),ax,ay,0);	 %Arrow with the robot orientation at each pose
			set(h, 'color',color);
		end
	end
  
  hold off
  