% PLOTDATA3D
%
%% Description
% Author -
%     Ernesto Homar Teniente Aviles
%     CONACYT Scholarship
%     Institut de Robotica i Informatica Industrial, CSIC-UPC
%     Last change: 11/2010
%
% Usage:
%   plotData3D(data[,OPTIONS])
%
% Variables
%  Inputs:
%     data - 3xn matrix, 
%
%   Options:
%     'Color'     -
%     'Subfigure' -
%     'BGC'       -
%     'FigName'   -
%     'PlotName'  -
%     'FigureNumber'-
%     'MarkerSize'  -
%     'DisplayName' -

%% CODE

function fig = plotData3D(data,varargin)

chkvarargin(varargin,2);


% Process arguments
[color,     ...
 subFig,    ...
 bgc,       ...
 figName,   ...
 plotName,  ...
 figIDn,    ...
 markerSize,...
 id,         ...
 marker] = process_options(varargin, 'Color', 'b',    ...
                      'Subfigure', 0,       ...
			                'BGC', 0,             ...
                      'FigName','Plot 3D',  ...
                      'PlotName','Scan 3D points', ...
                      'FigureNumber', 0,    ...
                      'MarkerSize', 4,      ...
                      'DisplayName',[],     ...
                      'Marker','.');

% Figure positioning
scrsz = get(0,'ScreenSize');
figpose = [scrsz(3)/4 scrsz(4)/6 scrsz(3)/2 2*scrsz(4)/3];


% Start figure
if figIDn
  fig = figure(figIDn);
else
  fig = figure(1);
end


No = sprintf(' %d',figIDn);
name1 = [figName ', Fig : ' No];

set(fig,'Name',name1,'NumberTitle','off');
% set(fig,'Name',name1,'NumberTitle','off','Position',figpose);

% Sobfigure enable
if subFig
    subplot(subFig(1),subFig(2),subFig(3))
end

if bgc
  whitebg(bgc);
end
symbol = '.';

hold on
% view(0,90);
h = plot3(0,0,0,symbol,'EraseMode','normal', ...
          'MarkerSize',markerSize,        ...
          'DisplayName',id,             ...
          'Color', color, ...
          'Marker', marker) ;
title(plotName);
set(h,'XData',data(1,:),'YData',data(2,:),'ZDATA',data(3,:)); % 
axis equal
