%% Plot 3D Poses
%  Plots a pose given a vector with (x,y,z) or a vector containing
% (x,y,z,rx,ry,rz)
%
%% Description
%
% Author -
%           Ernesto Homar Teniente Aviles
% 			CONACYT Fellow
%			Institut de Robotica i Informatica Industrial, CSIC-UPC
%           Last change - May, 2008
%
% This code has been developed as part of the EU URUS Project
% FP6-IST-045062
%
% Usage:
%   plotPoses(poses[, OPTIONS]);
%
% Variables
%
% Inputs :
% 	poses - 3xn , it should have (x,y,z) values
%         - 6xn matrix, it should have (x,y,z,rx,ry,rz) values
%					* rx - roll, rotation angle about the x axis
%					* ry - pitch, rotation angle about the y axis
%					* rz - yaw, rotation angle about the z axis
%
% Options:
%   'PoseColor'   - a string 
%   'PoseNumber'  - a number, containing the text to plot with the pose
% 	'FigureNumber'- an integer, figure to plot the pose
%   'PoseScale'   - +real, axis orientation scale
%

%% Code:
function plotPoses3D(poses,varargin)

% Varargin check
chkvarargin(varargin,2);

[posecolor  ...
 posescale  ...
 pose_n     ...
 markersize ...
 fig_n] =process_options(varargin,...
                'PoseColor','y.',...
                'PoseScale', 0.5,...
                'poseNumber',1,...
                'MarkerSize', 15,...
                'FigureNumber', []);

if ~isempty(fig_n)
	figure(fig_n);
end
hold on;
plot3(poses(1,:),poses(2,:),poses(3,:),posecolor,'erasemode',...
        'normal','MarkerSize',markersize)

for i = 1:size(poses,2)
    if size(poses,1) > 3
      drawaxis([rxryrz2R(poses(4,i),poses(5,i),poses(6,i)) ...
            poses(1:3,i);0 0 0 1],posescale,pose_n);
    else
      text((poses(1,i)+posescale/4),poses(2,i),poses(3,i)+posescale/4,int2str(pose_n));
    end
%     if drawnow
%         drawnow;
%     end
    pose_n = pose_n+1;
end